using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.Xml;
using System.IO;
using WindowsProxy;
using System.Threading;
using System.Diagnostics;

namespace Squipy
{
    public partial class FormSquipy : Form
    {
        private String version = "1.2";

        private delegate void updateUILimpiar();
        private delegate void updateUICopiar();
        private delegate void updateUIDatos(string valor, string tipo);
        private delegate void updateUIRespuesta(string cabecera, string cuerpo);
        private delegate void updateUIPost(string valor);
        private delegate void updateUIFrente();
        private delegate void updateUIConectar(bool keepalive, HttpWebRequest webRequest, string metodo, string url);
        private delegate void updateUIEnviar();
        private delegate void updateUIBotones(bool activo);
        private delegate void updateUIEstado(bool activo);
        private delegate void updateUIPostAGet();
        private delegate void updateUIArbol(String host, String pagina, String enviado, String cabecera, String cuerpo);
        private delegate void updateUIArbolDatos(int pos);

        private String sMetodo;
        private String sProtocolo;
        private String sUri;
        private String sHost;
        private String sAccept;
        private String sAcceptCharset;
        private String sAcceptLanguage;
        private String sAcceptEncoding;
        private String sUserAgent;
        private String sKeepAlive;
        private String sReferer;
        private String sProxyConnection;
        private String sCookie;
        private String sContentType;
        private String sPost;

        // navegacion con filtrado
        private HttpListener listener;
        private HttpListenerContext context;
        private CookieContainer ContenedorCookies;
        private Thread tareaEscucha;
        private bool filtrar = false;
        private byte[] bufferCuerpo = null;

        // arbol de logs
        private List<string> aUrl;
        private List<string> aHost;
        private List<string> aEnviado;
        private List<string> aCabecera;
        private List<string> aCuerpo;
        TreeView arbolTmp;

        // navegacion sin filtrado
        private HttpServer listenerSinFiltro;

        // configuraciones
        private String Puerto;
        private String Idioma;
        private String UserAgent;
        private String AcceptLanguage;
        private String ConvertirPG;
        private String ConvertirGP;
        private String ConvertirHex;
        private String FiltrarUA;
        private String FiltrarAL;
        private String Proxy;
        private String ActivarProxy;
        private String GPparams;
        private String cVState;
        private List<string> Extensiones;
        private List<string> Urls;

        public FormSquipy()
        {
            InitializeComponent();
        }

        #region Leer configuraciones
        private void leerConfiguracion()
        {
            if (!File.Exists("config.xml"))
            {
                // si no existe el fichero de configuracin, lo creamos
                XmlTextWriter f = new XmlTextWriter("config.xml", new System.Text.ASCIIEncoding());
                f.Formatting = Formatting.Indented;
                f.WriteStartDocument();
                f.WriteStartElement("configuracion");
                f.WriteElementString("puerto", "8080");
                f.WriteElementString("idioma", "es");
                f.WriteElementString("user-agent", "");
                f.WriteElementString("accept-language", "");
                f.WriteElementString("convertirPG", "no");
                f.WriteElementString("convertirGP", "no");
                f.WriteElementString("convertirHex", "no");
                f.WriteElementString("filtrarUA", "no");
                f.WriteElementString("filtrarAL", "no");
                f.WriteElementString("proxy", "");
                f.WriteElementString("activarProxy", "no");
                f.WriteElementString("GPparams", "Todos");
                f.WriteStartElement("extensiones");
                f.WriteEndElement();
                f.WriteStartElement("urls");
                f.WriteEndElement();
                f.WriteEndDocument();
                f.Close();
            }

            try
            {
                // leemos el fichero de configuracin
                XmlDocument xDoc = new XmlDocument();
                xDoc.Load("config.xml");
                XmlNodeList nodoPuerto = xDoc.GetElementsByTagName("puerto");
                try { Puerto = nodoPuerto.Item(0).InnerText; }
                catch { Puerto = "8080"; }

                XmlNodeList nodoIdioma = xDoc.GetElementsByTagName("idioma");
                try { Idioma = nodoIdioma.Item(0).InnerText; }
                catch { Idioma = "en"; }

                XmlNodeList nodoUserAgent = xDoc.GetElementsByTagName("user-agent");
                try { UserAgent = nodoUserAgent.Item(0).InnerText; }
                catch { UserAgent = ""; }

                XmlNodeList nodoAcceptLanguage = xDoc.GetElementsByTagName("accept-language");
                try { AcceptLanguage = nodoAcceptLanguage.Item(0).InnerText; }
                catch { AcceptLanguage = ""; }

                XmlNodeList nodoConvertirPG = xDoc.GetElementsByTagName("convertirPG");
                try { ConvertirPG = nodoConvertirPG.Item(0).InnerText; }
                catch { ConvertirPG = "no"; }

                XmlNodeList nodoConvertirGP = xDoc.GetElementsByTagName("convertirGP");
                try { ConvertirGP = nodoConvertirGP.Item(0).InnerText; }
                catch { ConvertirGP = "no"; }

                XmlNodeList nodoConvertirHex = xDoc.GetElementsByTagName("convertirHex");
                try { ConvertirHex = nodoConvertirHex.Item(0).InnerText; }
                catch { ConvertirHex = "no"; }

                XmlNodeList nodoFiltrarUA = xDoc.GetElementsByTagName("filtrarUA");
                try { FiltrarUA = nodoFiltrarUA.Item(0).InnerText; }
                catch { FiltrarUA = "no"; }

                XmlNodeList nodoFiltrarAL = xDoc.GetElementsByTagName("filtrarAL");
                try { FiltrarAL = nodoFiltrarAL.Item(0).InnerText; }
                catch { FiltrarAL = "no"; }

                XmlNodeList nodoProxy = xDoc.GetElementsByTagName("proxy");
                try { Proxy = nodoProxy.Item(0).InnerText; }
                catch { Proxy = ""; }

                XmlNodeList nodoActivarProxy = xDoc.GetElementsByTagName("activarProxy");
                try { ActivarProxy = nodoActivarProxy.Item(0).InnerText; }
                catch { ActivarProxy = "no"; }

                XmlNodeList nodoGPparams = xDoc.GetElementsByTagName("GPparams");
                try { GPparams = nodoGPparams.Item(0).InnerText; }
                catch { GPparams = "Todos"; }

                XmlNodeList nodoExtensiones = xDoc.GetElementsByTagName("extensiones");
                XmlNode nodoExtension = nodoExtensiones.Item(0);

                Extensiones = new List<string>(100);

                foreach (XmlElement nodo in nodoExtension)
                    Extensiones.Add(nodo.InnerText);

                XmlNodeList nodoUrls = xDoc.GetElementsByTagName("urls");
                XmlNode nodoUrl = nodoUrls.Item(0);

                Urls = new List<string>(100);

                foreach (XmlElement nodo in nodoUrl)
                    Urls.Add(nodo.InnerText);
            }
            catch
            {
                try { File.Delete("config.xml"); }
                catch { }

                leerConfiguracion();
            }
        }

        private void ConfiguraIdioma()
        {
            if (Idioma == "es")
            {
                archivoToolStripMenuItem.Text = "&Archivo";
                activarFiltroToolStripMenuItem.Text = "&Comenzar captura de datos";
                pararCapturaDeDatosToolStripMenuItem.Text = "&Parar captura de datos";
                salirToolStripMenuItem.Text = "&Salir";
                herramientasToolStripMenuItem.Text = "&Herramientas";
                decodificarViewStateDeASPToolStripMenuItem.Text = "&Decodificar ViewState de ASP";
                configuracinToolStripMenuItem.Text = "&Configuracin";
                filtrosToolStripMenuItem.Text = "&Filtros";
                puertoParaLaEscuchaToolStripMenuItem.Text = "&Puerto para la escucha";
                extensionesNoFiltradasToolStripMenuItem.Text = "&Extensiones no filtradas";
                urlsNoFiltradasToolStripMenuItem.Text = "&Urls no filtradas";
                idiomaToolStripMenuItem.Text = "&Idioma";
                ayudaToolStripMenuItem.Text = "&Ayuda";
                acercaDeToolStripMenuItem.Text = "&Acerca de";

                tabCapturas.Text = "Capturas";
                tabHistorico.Text = "Histrico";
                label1.Text = "Enviado";
                label2.Text = "Recibido";

                lmetodo.Text = "Mtodo:";
                lprotocolo.Text = "Protocolo:";
                lpost.Text = "Parmetros POST:";
                ldatosservidor.Text = "Datos recibidos del servidor:";
                lfiltrarex.Text = "Filtrar Extensiones";
                lfiltrarurl.Text = "Filtrar Urls";
                lfiltrartodo.Text = "Filtrar Todo";
                lcapturar.Text = "Capturar respuesta";
                lajustar.Text = "Ajustar";
                labelfiltrar.Text = "Filtrar Url:";
                label3.Text = "Url a guardar:";

                bCapturar.Text = "CAPTURAR";
                bParar.Text = "PARAR";
                bConvertir.Text = "CONVERTIR GET<->POST";
                bEnviar.Text = "ENVIAR SERVIDOR";
                bCancelar.Text = "NO ENVIAR";
                bContinuar.Text = "ENVIAR CLIENTE";
                bSalir.Text = "SALIR";
                bAnalizar.Text = "ANALIZAR";
                bLimpiar.Text = "LIMPIAR";
                bCargar.Text = "CARGAR";
                bGuardar.Text = "GUARDAR";
                bExportar.Text = "EXPORTAR";
                bOrdenar.Text = "FORMATEAR TEXTO";

                cGuardarUrls.Items[0] = "Todas";
                cGuardarUrls.Text = "Todas";
            }
            else
            {
                archivoToolStripMenuItem.Text = "&File";
                activarFiltroToolStripMenuItem.Text = "&Start data capture";
                pararCapturaDeDatosToolStripMenuItem.Text = "S&top data capture";
                salirToolStripMenuItem.Text = "&Exit";
                herramientasToolStripMenuItem.Text = "&Tools";
                decodificarViewStateDeASPToolStripMenuItem.Text = "&Decode ASP ViewState";
                configuracinToolStripMenuItem.Text = "&Config";
                filtrosToolStripMenuItem.Text = "&Filters";
                puertoParaLaEscuchaToolStripMenuItem.Text = "&Listen port";
                extensionesNoFiltradasToolStripMenuItem.Text = "Ignored &extensions";
                urlsNoFiltradasToolStripMenuItem.Text = "Ignored &urls";
                idiomaToolStripMenuItem.Text = "Lan&guage";
                ayudaToolStripMenuItem.Text = "&Help";
                acercaDeToolStripMenuItem.Text = "&About";

                tabCapturas.Text = "Captures";
                tabHistorico.Text = "Logs";
                label1.Text = "Send";
                label2.Text = "Received";

                lmetodo.Text = "Method:";
                lprotocolo.Text = "Protocol:";
                lpost.Text = "POST Params:";
                ldatosservidor.Text = "Received data from server:";
                lfiltrarex.Text = "Filter Extensions";
                lfiltrarurl.Text = "Filter Urls";
                lfiltrartodo.Text = "Filter All";
                lcapturar.Text = "Capture response";
                lajustar.Text = "WordWrap";
                labelfiltrar.Text = "Filter Url:";
                label3.Text = "Url to save:";

                bCapturar.Text = "CAPTURE";
                bParar.Text = "STOP";
                bConvertir.Text = "CONVERT GET<->POST";
                bEnviar.Text = "SEND SERVER";
                bCancelar.Text = "DONT SEND";
                bContinuar.Text = "SEND CLIENT";
                bSalir.Text = "EXIT";
                bAnalizar.Text = "ANALYZE";
                bLimpiar.Text = "CLEAR";
                bCargar.Text = "LOAD";
                bGuardar.Text = "SAVE";
                bExportar.Text = "EXPORT";
                bOrdenar.Text = "FORMAT TEXT";

                cGuardarUrls.Items[0] = "All";
                cGuardarUrls.Text = "All";
            }
        }
        #endregion

        #region escribir datos en pantalla
        private void TraerAlFrente()
        {
            if (this.InvokeRequired == false)
            {
                tabControl1.SelectedIndex = 1;
                this.Activate();
                this.Focus();
            }
            else
            {
                updateUIFrente updateMethod = new updateUIFrente(this.TraerAlFrente);
                this.Invoke(updateMethod, new object[] { });
            }
        }

        private void ActualizaDatosArbol(int pos)
        {
            if (this.InvokeRequired == false)
            {
                if (pos == -1)
                {
                    cLogEnviado.Text = "";
                    cLogCabecera.Text = "";
                    cLogCuerpo.Text = "";
                }
                else
                {
                    try
                    {
                        cLogEnviado.Text = aEnviado[pos];
                        cLogCabecera.Text = aCabecera[pos];
                        cLogCuerpo.Text = aCuerpo[pos];
                    }
                    catch { }
                }
            }
            else
            {
                updateUIArbolDatos updateMethod = new updateUIArbolDatos(this.ActualizaDatosArbol);
                this.Invoke(updateMethod, new object[] { pos });
            }
        }

        private void CopiarEnArbol(String host, String pagina, String enviado, String cabecera, String cuerpo)
        {
            if (this.InvokeRequired == false)
            {
                // revisamos el host
                tree.SelectedNode = tree.Nodes.Find("/", false)[0];

                TreeNode[] x = tree.Nodes.Find(host, true);

                if (x.Length == 0)
                {
                    tree.SelectedNode.Nodes.Add(host, host, 1);
                    cGuardarUrls.Items.Add(host);
                }

                x = tree.Nodes.Find(pagina.Replace("http//", ""), true);

                if ((x.Length == 0) || (!pagina.Contains(".js") && !pagina.Contains(".css") && !pagina.Contains(".gif")
                     && !pagina.Contains(".jpg") && !pagina.Contains(".png") && !pagina.Contains(".ico")))
                {
                    // revisamos la url
                    tree.SelectedNode = tree.Nodes.Find(host, true)[0];
                    tree.SelectedNode.Nodes.Add(pagina.Replace("http//", ""), pagina.Replace("http//", ""), 3);

                    aHost.Add(host);
                    aUrl.Add(pagina);
                    aEnviado.Add(enviado);
                    aCabecera.Add(cabecera);
                    aCuerpo.Add(cuerpo);
                }
            }
            else
            {
                updateUIArbol updateMethod = new updateUIArbol(this.CopiarEnArbol);
                this.Invoke(updateMethod, new object[] { host, pagina, enviado, cabecera, cuerpo });
            }
        }

        private void LimpiarDatos()
        {
            if (this.InvokeRequired == false)
            {
                cMetodo.Text = "GET";
                cProtocolo.Text = "";
                cUri.Text = "";
                cHost.Text = "";
                cAccept.Text = "";
                cAcceptCharset.Text = "";
                cAcceptLanguage.Text = "";
                cAcceptEncoding.Text = "";
                cUserAgent.Text = "";
                cKeepAlive.Text = "";
                cReferer.Text = "";
                cProxyConnection.Text = "";
                cCookie.Text = "";
                cContentType.Text = "";
                cPost.Text = "";
                cCabecera.Text = "";
                cCuerpo.Text = "";
                bOrdenar.Enabled = false;

                sMetodo = "";
                sProtocolo = "";
                sUri = "";
                sHost = "";
                sAccept = "";
                sAcceptCharset = "";
                sAcceptLanguage = "";
                sAcceptEncoding = "";
                sUserAgent = "";
                sKeepAlive = "";
                sReferer = "";
                sProxyConnection = "";
                sCookie = "";
                sContentType = "";
                sPost = "";
                
                bContinuar.Enabled = false;
                cCabecera.Enabled = false;
                cCuerpo.Enabled = false;
            }
            else
            {
                updateUILimpiar updateMethod = new updateUILimpiar(this.LimpiarDatos);
                this.Invoke(updateMethod, new object[] { });
            }
        }

        private void ActualizarEstado(bool activo)
        {
            if (this.InvokeRequired == false)
            {
                bCapturar.Enabled = !activo;
                bParar.Enabled = activo;
            }
            else
            {
                updateUIEstado updateMethod = new updateUIEstado(this.ActualizarEstado);
                this.Invoke(updateMethod, new object[] { activo });
            }
        }

        private void ActualizarBotones(bool activo)
        {
            if (this.InvokeRequired == false)
            {
                bEnviar.Enabled = activo;
                bCancelar.Enabled = activo;
                cCabecera.Enabled = !activo;
                cCuerpo.Enabled = !activo;

                bContinuar.Enabled = false;

                if ((bEnviar.Enabled == false) && (lcapturar.Checked == true))
                    bContinuar.Enabled = true;

                if (bEnviar.Enabled == true)
                {
                    cCabecera.Text = "";
                    cCuerpo.Text = "";
                    bConvertir.Enabled = true;
                }
                else
                    bConvertir.Enabled = false;

                if (cCuerpo.Text != "" && lcapturar.Checked) bOrdenar.Enabled = true;
                else bOrdenar.Enabled = false;

                cMetodo.Enabled = activo;
                cUri.Enabled = activo;
                cHost.Enabled = activo;
                cAccept.Enabled = activo;
                cAcceptCharset.Enabled = activo;
                cAcceptLanguage.Enabled = activo;
                cAcceptEncoding.Enabled = activo;
                cUserAgent.Enabled = activo;
                cKeepAlive.Enabled = activo;
                cReferer.Enabled = activo;
                cProxyConnection.Enabled = activo;
                cCookie.Enabled = activo;
                cContentType.Enabled = activo;
                cPost.Enabled = activo;
                bDesencriptaVS.Enabled = false;
            }
            else
            {
                updateUIBotones updateMethod = new updateUIBotones(this.ActualizarBotones);
                this.Invoke(updateMethod, new object[] { activo });
            }
        }

        private void ActualizarDatos(string valor, string tipo)
        {
            if (this.InvokeRequired == false)
            {
                if (tipo == "metodo") { cMetodo.Text = valor; sMetodo = valor; }
                if (tipo == "uri") { cUri.Text = valor; sUri = valor; }
                if (tipo == "protocolo") { cProtocolo.Text = valor; sProtocolo = valor; }
                if (tipo == "host") { cHost.Text = valor; sHost = valor; }
                if (tipo == "accept") { cAccept.Text = valor; sAccept = valor; }
                if (tipo == "accept-language") { cAcceptLanguage.Text = valor; sAcceptLanguage = valor; }
                if (tipo == "accept-encoding") { cAcceptEncoding.Text = valor; sAcceptEncoding = valor; }
                if (tipo == "accept-charset") { cAcceptCharset.Text = valor; sAcceptCharset = valor; }
                if (tipo == "keep-alive") { cKeepAlive.Text = valor; sKeepAlive = valor; }
                if (tipo == "proxy-connection") { cProxyConnection.Text = valor; sProxyConnection = valor; }
                if (tipo == "cookie") { cCookie.Text = valor; sCookie = valor; }
                if (tipo == "referer") { cReferer.Text = valor; sReferer = valor; }
                if (tipo == "user-agent") { cUserAgent.Text = valor; sUserAgent = valor; }
                if (tipo == "content-type") { cContentType.Text = valor; sContentType = valor; }
            }
            else
            {
                updateUIDatos updateMethod = new updateUIDatos(this.ActualizarDatos);
                this.Invoke(updateMethod, new object[] { valor, tipo });
            }
        }

        private void CopiarDatos()
        {
            if (this.InvokeRequired == false)
            {
                sMetodo = cMetodo.Text;
                sUri = cUri.Text;
                sProtocolo = cProtocolo.Text;
                sHost = cHost.Text;
                sAccept = cAccept.Text;
                sAcceptLanguage = cAcceptLanguage.Text;
                sAcceptEncoding = cAcceptEncoding.Text;
                sAcceptCharset = cAcceptCharset.Text;
                sKeepAlive = cKeepAlive.Text;
                sProxyConnection = cProxyConnection.Text;
                sCookie = cCookie.Text;
                sReferer = cReferer.Text;
                sUserAgent = cUserAgent.Text;
                sContentType = cContentType.Text;
                sPost = cPost.Text;
            }
            else
            {
                updateUICopiar updateMethod = new updateUICopiar(this.CopiarDatos);
                this.Invoke(updateMethod, new object[] { });
            }
        }

        private void ActualizarRespuesta(string cabecera, string cuerpo)
        {
            if (this.InvokeRequired == false)
            {
                if (cabecera != "") cCabecera.Text = cabecera;
                if (cuerpo != "") cCuerpo.Text = cuerpo;

                if (cCuerpo.Text != "" && lcapturar.Checked) bOrdenar.Enabled = true;
                else bOrdenar.Enabled = false;

                if (cuerpo.ToUpper().Contains("__VIEWSTATE"))
                {
                    try
                    {
                        String tmp = cuerpo;
                        int pos = tmp.ToUpper().IndexOf("__VIEWSTATE") + 11;
                        tmp = tmp.Substring(pos);
                        pos = tmp.Trim().ToUpper().IndexOf("VALUE=\"") + 7;
                        tmp = tmp.Substring(pos);
                        pos = tmp.IndexOf("\"");
                        tmp = tmp.Substring(0, pos);

                        cVState = tmp;
                        bDesencriptaVS.Enabled = true;
                    }
                    catch { bDesencriptaVS.Enabled = false; }
                }
                else
                    bDesencriptaVS.Enabled = false;
            }
            else
            {
                updateUIRespuesta updateMethod = new updateUIRespuesta(this.ActualizarRespuesta);
                this.Invoke(updateMethod, new object[] { cabecera, cuerpo });
            }
        }

        private void ActualizarPost(string valor)
        {
            if (this.InvokeRequired == false)
            {
                cPost.Text = valor;
                sPost = cPost.Text;
            }
            else
            {
                updateUIPost updateMethod = new updateUIPost(this.ActualizarPost);
                this.Invoke(updateMethod, new object[] { valor });
            }
        }
        #endregion

        private void FormSquipy_Load(object sender, EventArgs e)
        {
            this.Text = "Squipy v" + version;
            this.ActualizarEstado(false);
            cProtocolo.Enabled = false;

            cGuardarUrls.Items.Clear();
            cGuardarUrls.Items.Add("Todas");
            cGuardarUrls.Text="Todas";

            DirectoryInfo dir = new DirectoryInfo(Path.GetDirectoryName(Application.ExecutablePath) + "\\data\\");
            if (!dir.Exists) dir.Create();

            dir = new DirectoryInfo(Path.GetDirectoryName(Application.ExecutablePath) + "\\data\\capture\\");
            if (!dir.Exists) dir.Create();

            ContenedorCookies = new CookieContainer();

            arbolTmp = new TreeView();
            tree.Nodes.Clear();
            tree.Nodes.Add("/", "/", 1);

            aUrl = new List<string>(1000);
            aHost = new List<string>(1000);
            aEnviado = new List<string>(1000);
            aCabecera = new List<string>(1000);
            aCuerpo = new List<string>(1000);

            lfiltrarex.Checked = true;
            lfiltrarurl.Checked = true;
            lfiltrartodo.Checked = false;
            lajustar.Checked = false;

            leerConfiguracion();
            ConfiguraIdioma();

            Estado(false);
            ActualizarBotones(false);
            bContinuar.Enabled = false;
            cCabecera.Enabled = false;
            cCuerpo.Enabled = false;

            if (Actualizaciones() == true)
            {
                if (Idioma == "es")
                    MessageBox.Show("Hay disponible una nueva versin", "Squipy", MessageBoxButtons.OK, MessageBoxIcon.Information);
                else
                    MessageBox.Show("There is a new version", "Squipy", MessageBoxButtons.OK, MessageBoxIcon.Information);

                Process myproc;
                myproc = new Process();
                myproc.EnableRaisingEvents = false;
                ProcessStartInfo psi = new ProcessStartInfo("iexplore.exe", "http://www.pepelux.org/programs.php");
                myproc.StartInfo = psi;
                myproc.Start();
                myproc.WaitForInputIdle();
            }

            if (!HttpListener.IsSupported)
            {
                if (Idioma == "es")
                    MessageBox.Show("Es necesario Windows XP SP2 o superior", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                else
                    MessageBox.Show("Windows XP SP2 or higher is necessary", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);

                Close();
            }

            // arrancamos el proxy
            ArrancarProxySinFiltro();
            //ArrancarProxyConFiltro();
        }

        private bool Actualizaciones()
        {
            HttpWebRequest req;

            req = (HttpWebRequest)WebRequest.Create("http://www.pepelux.org/programs/squipyversion.txt");
            req.Method = "GET";
            req.UserAgent = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.1) Gecko/2008072820 Firefox/3.0.1";
            req.ContentType = "application/x-www-form-urlencoded";

            try
            {
                Stream objStream = req.GetResponse().GetResponseStream();
                StreamReader objReader = new StreamReader(objStream);

                String sLine = "";
                String cad = "";

                while (sLine != null)
                {
                    sLine = objReader.ReadLine();

                    if (sLine != null)
                        cad += sLine + Environment.NewLine;
                }

                if (cad.Replace("\r\n", "") == version)
                    return false;
                else
                    return true;
            }
            catch
            {
                return false;
            }
        }

        private void ArrancarProxySinFiltro()
        {
            listenerSinFiltro = new HttpServer();
            listenerSinFiltro.Contenedor = ContenedorCookies;

            String ua = "";
            String al = "";

            if (UserAgent != "" && FiltrarUA != "no") ua = UserAgent;
            if (AcceptLanguage != "" && FiltrarAL != "no") al = AcceptLanguage;

            if (ActivarProxy == "yes")
                listenerSinFiltro.Arrancar(Puerto, ua, al, Proxy);
            else
                listenerSinFiltro.Arrancar(Puerto, ua, al, "");
        }

        private void ArrancarProxyConFiltro()
        {
            String uriLocal = "http://*:" + Puerto + "/";

            listener = new HttpListener();
            listener.Prefixes.Add(uriLocal);
            listener.Start();

            tareaEscucha = new Thread(new ThreadStart(this.Escuchar));
            tareaEscucha.Start();
            //Escuchar();
        }

        private void Escuchar()
        {
            // bucle infinito
            while (true)
            {
                // esperamos hasta obtener una conexin
                context = listener.GetContext();

                // obtenemos los datos de la conexin
                HttpListenerRequest request = context.Request;

                // metemos en los textbox los datos capturados
                capturarDatos(request);

                if ((filtrar == true) && (lfiltrartodo.Checked == false) && (extensionValida(cUri.Text) && (urlValida(cUri.Text))) && ((cFiltro.Text == "") || (request.RawUrl.Contains(cFiltro.Text))))
                {
                    this.ActualizarBotones(true);

                    if ((cCookie.Text.ToUpper().Contains("__VIEWSTATE")) || (cPost.Text.ToUpper().Contains("__VIEWSTATE")))
                    {
                        String tmp;

                        if (cCookie.Text.ToUpper().Contains("__VIEWSTATE")) tmp = cCookie.Text;
                        else tmp = cPost.Text;

                        int pos = tmp.ToUpper().IndexOf("__VIEWSTATE") + 11;
                        tmp = tmp.Substring(pos);
                        pos = tmp.Trim().ToUpper().IndexOf("VALUE=\"") + 7;
                        tmp = tmp.Substring(pos);
                        pos = tmp.IndexOf("\"");
                        tmp = tmp.Substring(0, pos);

                        cVState = tmp;
                        bDesencriptaVS.Enabled = true;
                    }
                    else
                        bDesencriptaVS.Enabled = false;

                    this.TraerAlFrente();
                    tareaEscucha.Suspend();
                }
                else
                    ManejadorDePeticiones(context);
            }
        }

        private void capturarDatos(HttpListenerRequest request)
        {
            /**********************************************************************************/
            /***** Obtenemos los datos de la direccin y cabeceras que podremos modificar *****/
            /**********************************************************************************/

            // url destino
            if (ConvertirHex == "yes")
            {
                sUri = request.RawUrl.Replace("%27", "'");
                int posIni = sUri.IndexOf("'");

                if (posIni > 0)
                {
                    String tmp1 = sUri.Substring(0, posIni);
                    String tmp = sUri.Substring(posIni + 1);
                    int posFin = tmp.IndexOf("&");

                    if (posFin < 0) posFin = tmp.LastIndexOf("'");

                    if (posFin > 0)
                    {
                        String tmp2 = tmp.Substring(posFin + 1);
                        tmp = Hex(tmp.Substring(0, posFin));
                        sUri = tmp1 + tmp + tmp2;
                    }

                    this.ActualizarDatos(sUri, "uri");
                }
                else
                    this.ActualizarDatos(request.RawUrl, "uri");
            }
            else
                this.ActualizarDatos(request.RawUrl, "uri");

            // metodo (GET o POST)
            this.ActualizarDatos(request.HttpMethod, "metodo");

            if (request.HttpMethod == "POST")
            {
                Stream body = request.InputStream;
                Encoding encoding = request.ContentEncoding;
                StreamReader reader = new StreamReader(body, encoding);
                String post = reader.ReadToEnd();

                this.ActualizarPost(post);
            }
            else
                this.ActualizarPost("");

            // protocolo (HTTP/1.1)
            this.ActualizarDatos("HTTP/" + request.ProtocolVersion.Minor.ToString() + "." + request.ProtocolVersion.Major.ToString(), "protocolo");

            // header: host
            if (request.Headers.GetValues("Host") != null)
                this.ActualizarDatos(request.Headers.GetValues("Host")[0], "host");
            else
                this.ActualizarDatos("", "host");

            // header: keep-alive
            bool keepalive = request.KeepAlive;

            if (!string.IsNullOrEmpty(request.Headers["Connection"]) && request.Headers["Connection"].IndexOf("keep-alive", StringComparison.InvariantCultureIgnoreCase) >= 0 ||
                !string.IsNullOrEmpty(request.Headers["Proxy-Connection"]) && request.Headers["Proxy-Connection"].IndexOf("keep-alive", StringComparison.InvariantCultureIgnoreCase) >= 0)
                keepalive = true;

            this.ActualizarDatos(keepalive.ToString(), "keep-alive");

            // header: accept
            String accept = "";

            if (request.Headers.GetValues("Accept") != null)
            {
                for (int i = 0; i < request.Headers.GetValues("Accept").Length; i++)
                {
                    if (accept != "") accept += "; ";
                    accept += request.Headers.GetValues("Accept")[i];
                }
            }

            this.ActualizarDatos(accept, "accept");

            // header: content-type
            if (request.ContentType != null) this.ActualizarDatos(request.ContentType, "content-type");

            // header: accept-charset
            String acceptCharset = "";

            if (request.Headers.GetValues("Accept-Charset") != null)
            {
                for (int i = 0; i < request.Headers.GetValues("Accept-Charset").Length; i++)
                {
                    if (acceptCharset != "") acceptCharset += "; ";
                    acceptCharset += request.Headers.GetValues("Accept-Charset")[i];
                }
            }

            this.ActualizarDatos(acceptCharset, "accept-charset");

            // header: accept-encoding
            String acceptEncoding = "";

            if (request.Headers.GetValues("Accept-Encoding") != null)
            {
                for (int i = 0; i < request.Headers.GetValues("Accept-Encoding").Length; i++)
                {
                    if (acceptEncoding != "") acceptEncoding += "; ";
                    acceptEncoding += request.Headers.GetValues("Accept-Encoding")[i];
                }
            }

            this.ActualizarDatos(acceptEncoding, "accept-encoding");

            // header: accept-language
            String acceptLanguage = "";

            if (AcceptLanguage != "" && FiltrarAL != "no")
                this.ActualizarDatos(AcceptLanguage, "accept-language");
            else
            {
                if (request.Headers.GetValues("Accept-Language") != null)
                {
                    for (int i = 0; i < request.Headers.GetValues("Accept-Language").Length; i++)
                    {
                        if (acceptLanguage != "") acceptLanguage += "; ";
                        acceptLanguage += request.Headers.GetValues("Accept-Language")[i];
                    }
                }

                this.ActualizarDatos(acceptLanguage, "accept-language");
            }

            // header: cookie
            String cookie = "";

            if (request.Headers.GetValues("Cookie") != null)
            {
                for (int i = 0; i < request.Headers.GetValues("Cookie").Length; i++)
                {
                    if (cookie != "") cookie += "; ";
                    cookie += request.Headers.GetValues("Cookie")[i];
                }
            }

            CookieCollection c = new CookieCollection();
            c = ContenedorCookies.GetCookies(new Uri (request.RawUrl));

            if (c.Count == 0)
                this.ActualizarDatos(cookie, "cookie");
            else
            {
                cookie = "";

                for (int i = 0; i < c.Count; i++)
                    cookie += c[i] + ";";

                this.ActualizarDatos(cookie.Substring(0, cookie.Length - 1), "cookie");
            }

            // header: proxy-connection
            if (request.Headers.GetValues("Proxy-Connection") != null)
                this.ActualizarDatos(request.Headers.GetValues("Proxy-Connection")[0], "proxy-connection");
            else
                this.ActualizarDatos("", "proxy-connection");

            // header: referer
            if (request.UrlReferrer != null) this.ActualizarDatos(request.UrlReferrer.AbsolutePath, "referer");
            else this.ActualizarDatos("", "referer");

            // header: user-agent
            if (UserAgent != "" && FiltrarUA != "no")
                this.ActualizarDatos(UserAgent, "user-agent");
            else
                this.ActualizarDatos(request.UserAgent, "user-agent");

            if (((request.HttpMethod == "POST") && (ConvertirPG == "yes")) || ((request.HttpMethod == "GET") && (ConvertirGP == "yes")))
                PostAGet();

            /**********************************************************************************/
        }

        private void ManejadorDePeticiones(HttpListenerContext context)
        {
            HttpListenerRequest req = context.Request;
            HttpListenerResponse res = context.Response;

            bool keepalive = req.KeepAlive;

            if (!string.IsNullOrEmpty(req.Headers["Connection"]) && req.Headers["Connection"].IndexOf("keep-alive", StringComparison.InvariantCultureIgnoreCase) >= 0 ||
                !string.IsNullOrEmpty(req.Headers["Proxy-Connection"]) && req.Headers["Proxy-Connection"].IndexOf("keep-alive", StringComparison.InvariantCultureIgnoreCase) >= 0)
                keepalive = true;

            ProcesarPeticionRemota(context, keepalive);
        }

        private void ProcesarPeticionRemota(HttpListenerContext context, bool keepalive)
        {
            //bool redireccion = false;
            //String ruta = "";
            String cookie = "";

            HttpListenerRequest req = context.Request;
            HttpListenerResponse res = context.Response;

            HttpWebRequest webRequest = CrearRespuesta(context, keepalive, "");

            //if (req.HasEntityBody)
            //    Relay(req.InputStream, webRequest.GetRequestStream());

            HttpWebResponse webResponse = null;

            try { webResponse = webRequest.GetResponse() as HttpWebResponse; }
            catch (WebException e) { webResponse = e.Response as HttpWebResponse; }

            if (webResponse == null)
            {
                EnviarRespuesta(context, 502, "Bad Gateway", keepalive, null);
                return;
            }

            //if (webResponse.StatusCode == HttpStatusCode.Found) redireccion = true;

            res.ProtocolVersion = HttpVersion.Version11;
            res.StatusCode = (int)webResponse.StatusCode;
            res.StatusDescription = webResponse.StatusDescription;

            res.KeepAlive = keepalive;

            for (int i = 0; i < webResponse.Headers.Count; i++)
            {
                string name = webResponse.Headers.GetKey(i).ToLower();
                string value = webResponse.Headers.Get(i).ToLower();

                //if (name == "location") ruta = value;
                if (name == "set-cookie")
                {
                    cookie = value;
                    ContenedorCookies.Add(webResponse.Cookies);
                }

                switch (name)
                {
                    case "content-length":
                        res.ContentLength64 = webResponse.ContentLength;
                        break;
                    case "transfer-encoding":
                        res.SendChunked = value.IndexOf("chunked") >= 0 ? true : false;
                        break;
                    default:
                        try { res.Headers.Set(name, value); }
                        catch { }
                        break;
                }
            }

            #region datos enviados
            String enviado = sMetodo + " " + cUri.Text.Replace(webResponse.ResponseUri.Host, "").Replace("http://", "") + " " + sProtocolo + Environment.NewLine;
            enviado += "Host: " + sHost + Environment.NewLine;
            enviado += "User-Agent: " + sUserAgent + Environment.NewLine;
            enviado += "Accept: " + sAccept + Environment.NewLine;
            enviado += "Accept-Language: " + sAcceptLanguage + Environment.NewLine;
            enviado += "Accept-Encoding: " + sAcceptEncoding + Environment.NewLine;
            enviado += "Accept-Charset: " + sAcceptCharset + Environment.NewLine;
            enviado += "Keep-Alive: " + sKeepAlive + Environment.NewLine;
            enviado += "Referer: " + sReferer + Environment.NewLine;
            enviado += "Proxy-Connection: " + sProxyConnection + Environment.NewLine;
            enviado += "Cookie: " + sCookie + Environment.NewLine;
            enviado += "Content-Type: " + sContentType + Environment.NewLine;
            enviado += "Content-Length: " + sPost.Length.ToString() + Environment.NewLine + Environment.NewLine;
            enviado += "Content-Type: " + sPost + Environment.NewLine;
            #endregion

            #region datos respuesta
            String cabecera = "HTTP/" + webResponse.ProtocolVersion.Minor.ToString() + "." + webResponse.ProtocolVersion.Major.ToString() + " " + Convert.ToInt32(webResponse.StatusCode).ToString() + " " + webResponse.StatusDescription + Environment.NewLine;
            cabecera += "Server: " + webResponse.Server + Environment.NewLine;

            if (webResponse.GetResponseHeader("X-Powered-By") != "")
                cabecera += "X-Powered-By: " + webResponse.GetResponseHeader("X-Powered-By") + Environment.NewLine;

            cabecera += "Host: " + webResponse.ResponseUri.Host + Environment.NewLine;
            cabecera += "Date: " + webResponse.GetResponseHeader("Date") + Environment.NewLine;

            try
            {
                String dia = webResponse.LastModified.Day.ToString().PadLeft(2, '0');
                String mes = webResponse.LastModified.Month.ToString().PadLeft(2, '0');
                String ano = webResponse.LastModified.Year.ToString();
                String hor = webResponse.LastModified.Hour.ToString().PadLeft(2, '0');
                String min = webResponse.LastModified.Minute.ToString().PadLeft(2, '0');
                String sec = webResponse.LastModified.Second.ToString().PadLeft(2, '0');

                cabecera += "Last-Modified: " + webResponse.LastModified.DayOfWeek.ToString().Substring(0, 3) + ", " + dia + "/" + mes + "/" + ano + " " + hor + ":" + min + ":" + sec + Environment.NewLine;
            }
            catch { } 

            if (webResponse.ContentType != "")
                cabecera += "Content-Type: " + webResponse.ContentType + Environment.NewLine;

            if (webResponse.ContentEncoding != "")
                cabecera += "Content-Encoding: " + webResponse.ContentEncoding + Environment.NewLine;

            cabecera += "Content-Length: " + webResponse.ContentLength.ToString() + Environment.NewLine;

            if (webResponse.CharacterSet != "")
                cabecera += "Character-Set: " + webResponse.CharacterSet + Environment.NewLine;

            if (webResponse.GetResponseHeader("Cookie") != "")
                cabecera += Environment.NewLine + "Cookie: " + webResponse.GetResponseHeader("Cookie") + Environment.NewLine;
            else
                if (cookie != "") cabecera += Environment.NewLine + "Cookie: " + cookie + Environment.NewLine;
            #endregion

            if (/*lfiltrartodo.Checked == false &&*/ !req.RawUrl.EndsWith(".gif") && !req.RawUrl.EndsWith(".jpg") && !req.RawUrl.EndsWith(".png")
                && !req.RawUrl.EndsWith(".jpeg") && !req.RawUrl.EndsWith(".ico") && !req.RawUrl.EndsWith(".swf"))
            {
                String cad = "";

                try
                {
                    // obtenemos el cuerpo
                    HttpWebRequest reqTmp = webRequest;
                    Stream objStream = reqTmp.GetResponse().GetResponseStream();
                    StreamReader objReader = new StreamReader(objStream);

                    String sLine = "";

                    while (sLine != null)
                    {
                        sLine = objReader.ReadLine();

                        if (sLine != null)
                            cad += sLine + Environment.NewLine;
                    }
                }
                catch { }

                this.ActualizarRespuesta(cabecera, cad);

                if (!Urls.Contains("http://" + webRequest.RequestUri.Host))
                    CopiarEnArbol(webRequest.RequestUri.Host, webRequest.RequestUri.AbsoluteUri, enviado, cabecera, cad);

                EnviarCliente();
            }
            else
                Relay(webResponse.GetResponseStream(), res.OutputStream);

            webResponse.Close();

            //if (redireccion == true)
            //    webRequest = CrearRespuesta(context, keepalive, ruta);
        }

        private void EnviarCliente()
        {
            if (this.InvokeRequired == false)
            {
                try
                {
                    UTF8Encoding encoding = new UTF8Encoding();
                    bufferCuerpo = encoding.GetBytes(cCuerpo.Text);

                    HttpListenerResponse response = context.Response;

                    if (bufferCuerpo.Length > 0)
                    {
                        // Get a response stream and write the response to it.
                        response.ContentLength64 = bufferCuerpo.Length;
                        System.IO.Stream output = response.OutputStream;
                        output.Write(bufferCuerpo, 0, bufferCuerpo.Length);
                        output.Close();
                    }
                }
                catch { }
            }
            else
            {
                updateUIEnviar updateMethod = new updateUIEnviar(this.EnviarCliente);
                this.Invoke(updateMethod, new object[] { });
            }
        }

        private void Relay(Stream input, Stream output)
        {
            try
            {
                Encoding enc = Encoding.UTF8;
                byte[] buffer = new byte[4096];
                String cuerpo = "";

                while (true)
                {
                    int bytesRead = input.Read(buffer, 0, buffer.Length);
                    if (bytesRead == 0) break;
                    output.Write(buffer, 0, bytesRead);
                    cuerpo += enc.GetString(buffer);
                }

                input.Close();
                output.Close();
                this.ActualizarRespuesta("", cuerpo);
            }
            catch { }
        }

        private void EnviarRespuesta(HttpListenerContext context, int code, string description, bool keepalive, byte[] body)
        {
            context.Response.StatusCode = code;
            context.Response.StatusDescription = description;
            context.Response.ProtocolVersion = HttpVersion.Version11;
            context.Response.KeepAlive = keepalive;

            if (body != null)
            {
                context.Response.ContentType = "text/plain";
                context.Response.ContentLength64 = body.Length;

                context.Response.OutputStream.Write(body, 0, body.Length);
                context.Response.OutputStream.Close();
            }
            else
                context.Response.ContentLength64 = 0;
        }

        private HttpWebRequest CrearRespuesta(HttpListenerContext context, bool keepalive, String url)
        {
            ServicePointManager.Expect100Continue = false;
            HttpListenerRequest req = context.Request;
            HttpWebRequest webRequest;

            if (url == "") // no es una redireccin
            {
                webRequest = WebRequest.Create(sUri) as HttpWebRequest;
                webRequest.Method = sMetodo;
            }
            else // es una redireccin
            {
                webRequest = WebRequest.Create("http://" + req.Url.Host + url) as HttpWebRequest;
                webRequest.Method = "GET";
            }

            if (UserAgent != null)
                webRequest.UserAgent = UserAgent;
            else
                webRequest.UserAgent = sUserAgent;

            if (sContentType != "")
                webRequest.ContentType = sContentType;
            else
                webRequest.ContentType = "application/x-www-form-urlencoded";

            webRequest.Referer = sReferer;
            webRequest.ProtocolVersion = HttpVersion.Version11;
            webRequest.KeepAlive = keepalive;
            webRequest.AllowAutoRedirect = false;
            webRequest.Headers.Add("Cookie: " + sCookie);
            //webRequest.CookieContainer = ContenedorCookies;

            if (Proxy != "" && ActivarProxy == "yes")
                webRequest.Proxy = new WebProxy("http://" + Proxy + "/", true);

            for (int i = 0; i < req.Headers.Count; i++)
            {
                string nameComp = req.Headers.GetKey(i).ToLower();
                string name = req.Headers.GetKey(i);
                string value = req.Headers.Get(i);

                switch (nameComp)
                {
                    case "host":
                        break;
                    case "connection":
                    case "cookie":
                        break;
                    case "proxy-connection":
                        webRequest.KeepAlive = keepalive;
                        break;
                    case "referer":
                        webRequest.Referer = value;
                        break;
                    case "user-agent":
                        if (UserAgent != null) value = UserAgent;
                        webRequest.UserAgent = sUserAgent;
                        break;
                    case "accept":
                        webRequest.Accept = sAccept;
                        break;
                    case "content-length":
                        webRequest.ContentLength = req.ContentLength64;
                        break;
                    case "content-type":
                        if (sContentType == "")
                            webRequest.ContentType = sContentType;
                        else
                            webRequest.ContentType = "application/x-www-form-urlencoded";
                    break;
                    case "if-modified-since":
                        webRequest.IfModifiedSince = DateTime.Parse(value);
                        break;
                    default:
                        try
                        {
                            if (nameComp == "accept-language") value = sAcceptLanguage;
                            if (nameComp == "accept-charset") value = "";// sAcceptCharset;
                            if (nameComp == "accept-encoding") value = "";// sAcceptEncoding;

                            webRequest.Headers.Set(name, value);
                        }
                        catch { }
                        break;
                }
            }

            if (webRequest.Method == "POST")
            {
                try
                {
                    byte[] byteData = UTF8Encoding.UTF8.GetBytes(sPost);
                    webRequest.ContentLength = byteData.Length;

                    Stream postStream = webRequest.GetRequestStream();
                    postStream.Write(byteData, 0, byteData.Length);
                    postStream.Close();
                }
                catch { }
            }

            return webRequest;
        }

        private bool extensionValida(string uri)
        {
            // devuelve true si no hay que filtrar y false en otro caso
            if (lfiltrarex.Checked == false) return true;
            if (!uri.Contains(".")) return true;

            int pos = uri.LastIndexOf('.');
            String ext = uri.Substring(pos);

            for (int i = 0; i < Extensiones.Count; i++)
                if (ext.ToLower() == Extensiones[i].ToLower()) return false;

            return true;
        }

        private bool urlValida(string uri)
        {
            // devuelve true si no hay que filtrar y false en otro caso
            if (lfiltrarurl.Checked == false) return true;

            int pos = uri.LastIndexOf('.');
            String ext = uri.Substring(pos);

            for (int i = 0; i < Urls.Count; i++)
                if (uri.ToLower().StartsWith(Urls[i].ToLower())) return false;

            return true;
        }

        #region botones
        private void bSalir_Click(object sender, EventArgs e)
        {
            try { listenerSinFiltro.Parar(); }
            catch { }

            try { tareaEscucha.Resume(); }
            catch { }

            try { tareaEscucha.Abort(); }
            catch { }

            try { listener.Stop(); }
            catch { }

            Close();
        }

        private void bEnviar_Click(object sender, EventArgs e)
        {
            this.ActualizarBotones(false);
            this.CopiarDatos();
            ManejadorDePeticiones(context);

            if (lcapturar.Checked == false)
            {
                this.LimpiarDatos();

                try { tareaEscucha.Resume(); }
                catch { }
            }
        }

        private void bContinuar_Click(object sender, EventArgs e)
        {
            this.LimpiarDatos();

            try { tareaEscucha.Resume(); }
            catch { }
        }

        private void Estado(bool activo)
        {
            this.ActualizarEstado(activo);
            filtrar = activo;
        }

        private void bCapturar_Click(object sender, EventArgs e)
        {
            tabControl1.SelectedIndex = 1;
            ContenedorCookies = listenerSinFiltro.Contenedor;
            try { listenerSinFiltro.Parar(); }
            catch { }

            Estado(true);

            ArrancarProxyConFiltro();
        }

        private void bParar_Click(object sender, EventArgs e)
        {
            PararCapturaFiltrada();
        }

        private void PararCapturaFiltrada()
        {
            try
            {
                this.ActualizarBotones(false);
                this.CopiarDatos();
                ManejadorDePeticiones(context);
                this.LimpiarDatos();
            }
            catch { }

            try { tareaEscucha.Resume(); }
            catch { }

            try { tareaEscucha.Abort(); }
            catch { }

            try { listener.Stop(); }
            catch { }

            bContinuar.Enabled = false;
            cCabecera.Enabled = false;
            cCuerpo.Enabled = false;
            lcapturar.Checked = false;

            Estado(false);

            ArrancarProxySinFiltro();
        }

        private void bCancelar_Click(object sender, EventArgs e)
        {
            this.ActualizarBotones(false);
            this.LimpiarDatos();

            try { tareaEscucha.Resume(); }
            catch { }
        }

        private void bAnalizar_Click(object sender, EventArgs e)
        {
            if (cLogCuerpo.Text.Trim().Length > 0)
            {
                FormAnalisis fAnalisis = new FormAnalisis();
                fAnalisis.Idioma = Idioma;
                fAnalisis.Texto = cLogCuerpo.Text;
                fAnalisis.ShowDialog();
            }
            else
            {
                if (Idioma == "es")
                    MessageBox.Show("No hay datos a analizar", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                else
                    MessageBox.Show("There are nothing to analyze", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void bDesencriptaVS_Click(object sender, EventArgs e)
        {
            FormViewState fViewState = new FormViewState();
            fViewState.Vstate = cVState;
            fViewState.Idioma = Idioma;
            fViewState.ShowDialog();
        }

        private void bConvertir_Click(object sender, EventArgs e)
        {
            PostAGet();
        }

        private void PostAGet()
        {
            if (this.InvokeRequired == false)
            {
                if (cMetodo.Text == "GET")
                {
                    cMetodo.Text = "POST";

                    if (cUri.Text.Contains("?"))
                    {
                        int pos = cUri.Text.IndexOf('?');

                        if (GPparams == "Todos")
                        {
                            cPost.Text = cUri.Text.Substring(pos + 1);
                            cUri.Text = cUri.Text.Substring(0, pos);
                        }
                        else
                        {
                            string ruta = cUri.Text.Substring(0, pos);
                            string datos = cUri.Text.Substring(pos + 1);
                            string[] p = datos.Split('&');
                            int nParam = p.Length;
                            string nuevoPost = "";
                            string nuevoGet = "";
                            int cont = 0;
                            int np = nParam - Convert.ToInt32(GPparams);

                            foreach (string valor in p)
                            {
                                if (cont < np)
                                    nuevoGet += valor + "&";
                                else
                                    nuevoPost += valor + "&";

                                cont++;
                            }

                            cPost.Text = nuevoPost.Substring(0, nuevoPost.Length - 1);
                            cUri.Text = ruta + nuevoGet.Substring(0, nuevoGet.Length - 1);
                        }

                        sPost = cPost.Text;
                        sUri = cUri.Text;
                    }
                }
                else
                {
                    cMetodo.Text = "GET";

                    if (cUri.Text.Contains("?")) cUri.Text += "&" + cPost.Text;
                    else cUri.Text += "?" + cPost.Text;
                    cPost.Text = "";
                }
            }
            else
            {
                updateUIPostAGet updateMethod = new updateUIPostAGet(this.PostAGet);
                this.Invoke(updateMethod, new object[] { });
            }
        }
        #endregion

        #region barra de menu
        private void filtrosToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FormFiltros fFiltros = new FormFiltros();
            fFiltros.Puerto = Puerto;
            fFiltros.Idioma = Idioma;
            fFiltros.UserAgent = UserAgent;
            fFiltros.AcceptLanguage = AcceptLanguage;
            fFiltros.ConvertirPG = ConvertirPG;
            fFiltros.ConvertirGP = ConvertirGP;
            fFiltros.ConvertirHex = ConvertirHex;
            fFiltros.FiltrarUA = FiltrarUA;
            fFiltros.FiltrarAL = FiltrarAL;
            fFiltros.Proxy = Proxy;
            fFiltros.ActivarProxy = ActivarProxy;
            fFiltros.Param = GPparams;
            fFiltros.Extensiones = Extensiones;
            fFiltros.Urls = Urls;
            fFiltros.ShowDialog();

            leerConfiguracion();
        }

        private void acercaDeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FormAyuda fAyuda = new FormAyuda();
            fAyuda.Idioma = Idioma;
            fAyuda.ShowDialog();
        }

        private void puertoParaLaEscuchaToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FormPuerto fPuerto = new FormPuerto();
            fPuerto.Puerto = Puerto;
            fPuerto.Idioma = Idioma;
            fPuerto.UserAgent = UserAgent;
            fPuerto.AcceptLanguage = AcceptLanguage;
            fPuerto.ConvertirPG = ConvertirPG;
            fPuerto.ConvertirGP = ConvertirGP;
            fPuerto.ConvertirHex = ConvertirHex;
            fPuerto.FiltrarUA = FiltrarUA;
            fPuerto.FiltrarAL = FiltrarAL;
            fPuerto.Proxy = Proxy;
            fPuerto.ActivarProxy = ActivarProxy;
            fPuerto.Param = GPparams;
            fPuerto.Extensiones = Extensiones;
            fPuerto.Urls = Urls;
            fPuerto.ShowDialog();

            if (fPuerto.Puerto != Puerto)
            {
                leerConfiguracion();

                try { listenerSinFiltro.Parar(); }
                catch { }

                try { tareaEscucha.Resume(); }
                catch { }

                try { tareaEscucha.Abort(); }
                catch { }

                try { listener.Stop(); }
                catch { }

                if (Idioma == "es")
                    MessageBox.Show("Se ha cambiado el puerto del proxy.\r\nActualiza la configuracin del navegador.", "Aviso", MessageBoxButtons.OK, MessageBoxIcon.Information);
                else
                    MessageBox.Show("The port of the proxy has changed.\r\nCheck your navigator config.", "Advise", MessageBoxButtons.OK, MessageBoxIcon.Information);

                ArrancarProxySinFiltro();
            }
        }

        private void extensionesNoFiltradasToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FormExtensiones fExtensiones = new FormExtensiones();
            fExtensiones.Puerto = Puerto;
            fExtensiones.Idioma = Idioma;
            fExtensiones.UserAgent = UserAgent;
            fExtensiones.AcceptLanguage = AcceptLanguage;
            fExtensiones.ConvertirPG = ConvertirPG;
            fExtensiones.ConvertirGP = ConvertirGP;
            fExtensiones.ConvertirHex = ConvertirHex;
            fExtensiones.FiltrarUA = FiltrarUA;
            fExtensiones.FiltrarAL = FiltrarAL;
            fExtensiones.Proxy = Proxy;
            fExtensiones.ActivarProxy = ActivarProxy;
            fExtensiones.Param = GPparams;
            fExtensiones.Extensiones = Extensiones;
            fExtensiones.Urls = Urls;
            fExtensiones.ShowDialog();

            leerConfiguracion();
        }

        private void urlsNoFiltradasToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FormUrls fUrls = new FormUrls();
            fUrls.Puerto = Puerto;
            fUrls.Idioma = Idioma;
            fUrls.UserAgent = UserAgent;
            fUrls.AcceptLanguage = AcceptLanguage;
            fUrls.ConvertirPG = ConvertirPG;
            fUrls.ConvertirGP = ConvertirGP;
            fUrls.ConvertirHex = ConvertirHex;
            fUrls.FiltrarUA = FiltrarUA;
            fUrls.FiltrarAL = FiltrarAL;
            fUrls.Proxy = Proxy;
            fUrls.ActivarProxy = ActivarProxy;
            fUrls.Param = GPparams;
            fUrls.Extensiones = Extensiones;
            fUrls.Urls = Urls;
            fUrls.ShowDialog();

            leerConfiguracion();
        }

        private void idiomaToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FormIdioma fIdioma = new FormIdioma();
            fIdioma.Puerto = Puerto;
            fIdioma.Idioma = Idioma;
            fIdioma.UserAgent = UserAgent;
            fIdioma.AcceptLanguage = AcceptLanguage;
            fIdioma.ConvertirPG = ConvertirPG;
            fIdioma.ConvertirGP = ConvertirGP;
            fIdioma.ConvertirHex = ConvertirHex;
            fIdioma.FiltrarUA = FiltrarUA;
            fIdioma.FiltrarAL = FiltrarAL;
            fIdioma.Proxy = Proxy;
            fIdioma.ActivarProxy = ActivarProxy;
            fIdioma.Param = GPparams;
            fIdioma.Extensiones = Extensiones;
            fIdioma.Urls = Urls;
            fIdioma.ShowDialog();

            leerConfiguracion();
            ConfiguraIdioma();
        }

        private void proxyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FormProxy fProxy = new FormProxy();
            fProxy.Puerto = Puerto;
            fProxy.Idioma = Idioma;
            fProxy.UserAgent = UserAgent;
            fProxy.AcceptLanguage = AcceptLanguage;
            fProxy.ConvertirPG = ConvertirPG;
            fProxy.ConvertirGP = ConvertirGP;
            fProxy.ConvertirHex = ConvertirHex;
            fProxy.FiltrarUA = FiltrarUA;
            fProxy.FiltrarAL = FiltrarAL;
            fProxy.Proxy = Proxy;
            fProxy.ActivarProxy = ActivarProxy;
            fProxy.Param = GPparams;
            fProxy.Extensiones = Extensiones;
            fProxy.Urls = Urls;
            fProxy.ShowDialog();

            leerConfiguracion();
            ConfiguraIdioma();
        }

        private void decodificarViewStateDeASPToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FormViewState fViewState = new FormViewState();
            fViewState.Vstate = "";
            fViewState.Idioma = Idioma;
            fViewState.ShowDialog();
        }

        private void spiderToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (bCapturar.Enabled == false) PararCapturaFiltrada();

            FormSpider fSpider = new FormSpider();
            fSpider.CUrl = "";

            if (ActivarProxy == "yes")
                fSpider.CProxy = Proxy;

            fSpider.Idioma = Idioma;
            fSpider.ShowDialog();
        }

        private void activarFiltroToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try { listenerSinFiltro.Parar(); }
            catch { }

            ArrancarProxyConFiltro();

            Estado(true);
        }

        private void pararCapturaDeDatosToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                this.ActualizarBotones(false);
                this.CopiarDatos();
                ManejadorDePeticiones(context);
                this.LimpiarDatos();
            }
            catch { }

            try { tareaEscucha.Resume(); }
            catch { }

            try { tareaEscucha.Abort(); }
            catch { }

            try { listener.Stop(); }
            catch { }

            bContinuar.Enabled = false;
            cCabecera.Enabled = false;
            cCuerpo.Enabled = false;
            lcapturar.Checked = false;

            Estado(false);

            ArrancarProxySinFiltro();
        }
        #endregion

        public string Hex(string ascii)
        {
            string hex = "";
            foreach (char c in ascii)
            {
                int tmp = c;
                hex += String.Format("{0:x2}", (uint)System.Convert.ToUInt32(tmp.ToString()));
            }
            return "0x" + hex;
        }

        private void tree_AfterSelect(object sender, TreeViewEventArgs e)
        {
            String dato = tree.SelectedNode.Name;
            this.ActualizaDatosArbol(aUrl.IndexOf(dato));
        }

        private void cUri_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == 13)
                bEnviar_Click(sender, e);
        }

        private void tree_DoubleClick(object sender, EventArgs e)
        {
            //String dato = tree.SelectedNode.Name;
            //int x = aUrl.IndexOf(dato);

            //if (x > -1)
            //    this.ActualizaDatosArbol(x);

            if (cLogCuerpo.Text.Trim().Length > 0)
            {
                FormAnalisis fAnalisis = new FormAnalisis();
                fAnalisis.Idioma = Idioma;
                fAnalisis.Texto = cLogCuerpo.Text;
                fAnalisis.ShowDialog();
            }
        }

        private void bOrdenar_Click(object sender, EventArgs e)
        {
            Cursor.Current = Cursors.WaitCursor;
            cCuerpo = formatearTextos.formatear(cCuerpo);
            Cursor.Current = Cursors.Default;
        }

        private void lajustar_CheckedChanged(object sender, EventArgs e)
        {
            cCuerpo.WordWrap = lajustar.Checked;
        }

        private void lfiltrartodo_CheckedChanged(object sender, EventArgs e)
        {
            if (lfiltrartodo.Checked == true)
            {
                if (bEnviar.Enabled == true) bEnviar_Click(sender, e);
                if (bContinuar.Enabled == true) bContinuar_Click(sender, e);
            }
        }

        private void bLimpiar_Click(object sender, EventArgs e)
        {
            DialogResult resp;

            if (Idioma=="es")
                resp = MessageBox.Show("Seguro que quieres limpar el rbol y borrar los datos de capturas?", "Squipy", MessageBoxButtons.OKCancel, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2);
            else
                resp = MessageBox.Show("Are you sure you want to clear tree and delete captured data?", "Squipy", MessageBoxButtons.OKCancel, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2);

            if (resp == DialogResult.OK)
            {
                tree.Nodes.Clear();
                tree.Nodes.Add("/", "/", 1);

                aUrl = new List<string>(1000);
                aHost = new List<string>(1000);
                aEnviado = new List<string>(1000);
                aCabecera = new List<string>(1000);
                aCuerpo = new List<string>(1000);
            }
        }

        private void bGuardar_Click(object sender, EventArgs e)
        {
            if (aUrl.Count > 0)
            {
                if (cGuardarUrls.Text != "Todas" && cGuardarUrls.Text != "All")
                    saveFileDialog1.FileName = cGuardarUrls.Text + ".xml";
                else
                    if (cGuardarUrls.Items.Count == 2)
                        saveFileDialog1.FileName = cGuardarUrls.Items[1] + ".xml";
                    else
                        saveFileDialog2.FileName = "";

                saveFileDialog1.InitialDirectory = Path.GetDirectoryName(Application.ExecutablePath) + "\\data\\capture\\";
                saveFileDialog1.ShowDialog();
            }
            else
            {
                if (Idioma == "es")
                    MessageBox.Show("No hay datos que guardar", "Guardar datos", MessageBoxButtons.OK, MessageBoxIcon.Information);
                else
                    MessageBox.Show("There is no data to save", "Save data", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        private void saveFileDialog1_FileOk(object sender, CancelEventArgs e)
        {
            String fichero = saveFileDialog1.FileName;

            if (File.Exists(fichero))
            {
                try { File.Delete(fichero); }
                catch 
                {
                    if (Idioma == "es")
                        MessageBox.Show("El fichero est bloqueado y no se puede sobreescribir", "Guardar datos", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    else
                        MessageBox.Show("File is locked and can't overwrite it", "Save data", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
            }

            XmlDocument xDoc = new XmlDocument();
            String cadXml = "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>" + Environment.NewLine;
            cadXml += "<data>" + Environment.NewLine;
            cadXml += "<host>/</host>" + Environment.NewLine;
            cadXml += "<lista_urls>" + Environment.NewLine;

            for (int i = 0; i < aUrl.Count; i++)
            {
                if (aHost[i] == cGuardarUrls.Text || cGuardarUrls.Text == "Todas" || cGuardarUrls.Text == "All")
                {
                    cadXml += "<urls>" + Environment.NewLine;
                    cadXml += "<url>" + aHost[i] + "</url>" + Environment.NewLine;
                    cadXml += "<ruta>![CDATA[" + aUrl[i].Replace("&", "#amp;").Replace("<", "#lt;").Replace(">", "#gt;") + "]]</ruta>" + Environment.NewLine;
                    cadXml += "<enviado>![CDATA[" + aEnviado[i].Replace("&", "#amp;").Replace("<", "#lt;").Replace(">", "#gt;") + "]]</enviado>" + Environment.NewLine;
                    cadXml += "<cabecera>![CDATA[" + aCabecera[i].Replace("&", "#amp;").Replace("<", "#lt;").Replace(">", "#gt;") + "]]</cabecera>" + Environment.NewLine;
                    cadXml += "<cuerpo>![CDATA[" + aCuerpo[i].Replace("&", "#amp;").Replace("<", "#lt;").Replace(">", "#gt;") + "]]</cuerpo>" + Environment.NewLine;
                    cadXml += "</urls>" + Environment.NewLine;
                }
            }

            cadXml += "</lista_urls>" + Environment.NewLine;
            cadXml += "</data>";
            xDoc.LoadXml(cadXml);

            xDoc.Save(fichero);

            if (Idioma == "es")
                MessageBox.Show("Se ha guardado el fichero", "Guardar datos", MessageBoxButtons.OK, MessageBoxIcon.Information);
            else
                MessageBox.Show("Data file saved correctly", "Save data", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void bCargar_Click(object sender, EventArgs e)
        {
            openFileDialog1.InitialDirectory = Path.GetDirectoryName(Application.ExecutablePath) + "\\data\\capture\\";
            openFileDialog1.ShowDialog();
        }

        private void openFileDialog1_FileOk(object sender, CancelEventArgs e)
        {
            Cursor.Current = Cursors.WaitCursor;

            XmlDocument xDoc = new XmlDocument();
            xDoc.Load(openFileDialog1.FileName);

            XmlNodeList listaUrls = xDoc.GetElementsByTagName("lista_urls");
            XmlNodeList lista = ((XmlElement)listaUrls[0]).GetElementsByTagName("urls");

            aUrl = new List<string>();
            aEnviado = new List<string>();
            aCabecera = new List<string>();
            aCuerpo = new List<string>();

            tree.Nodes.Clear();
            tree.Nodes.Add("/", "/", 1);
            cGuardarUrls.Items.Clear();

            if (Idioma == "es")
            {
                cGuardarUrls.Items.Add("Todas");
                cGuardarUrls.Text = "Todas";
            }
            else
            {
                cGuardarUrls.Items.Add("All");
                cGuardarUrls.Text = "All";
            }

            foreach (XmlElement nodo in lista)
            {
                aHost.Add(nodo["url"].InnerText);
                aUrl.Add(nodo["ruta"].InnerText.Replace("![CDATA[", "").Replace("]]", "").Replace("#amp;", "&").Replace("#lt;", "<").Replace("#gt;", ">"));
                aEnviado.Add(nodo["enviado"].InnerText.Replace("![CDATA[", "").Replace("]]", "").Replace("#amp;", "&").Replace("#lt;", "<").Replace("#gt;", ">"));
                aCabecera.Add(nodo["cabecera"].InnerText.Replace("![CDATA[", "").Replace("]]", "").Replace("#amp;", "&").Replace("#lt;", "<").Replace("#gt;", ">"));
                aCuerpo.Add(nodo["cuerpo"].InnerText.Replace("![CDATA[", "").Replace("]]", "").Replace("#amp;", "&").Replace("#lt;", "<").Replace("#gt;", ">"));

                Actualizar(nodo["url"].InnerText, nodo["ruta"].InnerText.Replace("![CDATA[", "").Replace("]]", "").Replace("#amp;", "&").Replace("#lt;", "<").Replace("#gt;", ">"));
            }

            Cursor.Current = Cursors.Default;
        }

        private void Actualizar(string host, string url)
        {
            TreeNode[] x = tree.Nodes.Find(host, true);

            if (host.Length > 0 && x.Length == 0)
            {
                TreeNode[] y = tree.Nodes.Find("/", true);
                this.AnadirNodo(y, host, host, true);
                cGuardarUrls.Items.Add(host);

                y = tree.Nodes.Find(host, true);
                this.AnadirNodo(y, url, url, false);
            }
            else
                this.AnadirNodo(x, url, url, false);
        }

        private void AnadirNodo(TreeNode[] y, string key, string dato, bool dir)
        {
            tree.SelectedNode = y[0];

            if (dir == true)
                tree.SelectedNode.Nodes.Add(key, dato, 1);
            else
                tree.SelectedNode.Nodes.Add(key, dato, 3);

            try { tree.Sort(); }
            catch { }

            try { tree.ExpandAll(); }
            catch { }
        }

        private void bExportar_Click(object sender, EventArgs e)
        {
            if (aUrl.Count > 0)
            {
                if (cGuardarUrls.Text != "Todas" && cGuardarUrls.Text != "All")
                    saveFileDialog2.FileName = cGuardarUrls.Text + ".html";
                else
                    if (cGuardarUrls.Items.Count == 2)
                        saveFileDialog2.FileName = cGuardarUrls.Items[1] + ".html";
                    else
                        saveFileDialog2.FileName = "";

                saveFileDialog2.InitialDirectory = Path.GetDirectoryName(Application.ExecutablePath) + "\\data\\capture\\";
                saveFileDialog2.ShowDialog();
            }
            else
            {
                if (Idioma == "es")
                    MessageBox.Show("No hay datos que guardar", "Guardar datos", MessageBoxButtons.OK, MessageBoxIcon.Information);
                else
                    MessageBox.Show("There is no data to save", "Save data", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        private void saveFileDialog2_FileOk(object sender, CancelEventArgs e)
        {
            Cursor.Current = Cursors.WaitCursor;

            String fichero = saveFileDialog2.FileName;

            if (File.Exists(fichero))
            {
                try { File.Delete(fichero); }
                catch
                {
                    if (Idioma == "es")
                        MessageBox.Show("El fichero est bloqueado y no se puede sobreescribir", "Guardar datos", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    else
                        MessageBox.Show("File is locked and can't overwrite it", "Save data", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
            }

            String cadHtml = "<html><head>";

            if (Idioma == "es")
                cadHtml += "<title>Informe</title></head>";
            else
                cadHtml += "<title>Report</title></head>";

            cadHtml += "<body>";
            cadHtml += "<table width='100%' border=1 cellpadding='5' cellspacing='5' bordercolor='#000000' bgcolor='#CCCCCC'>";

            if (Idioma == "es")
                cadHtml += "<tr><td><font face='Verdana, Arial, Helvetica, sans-serif'><b><h1>Informe guardado el: " + DateTime.Now.Day.ToString() + "/" + DateTime.Now.Month.ToString() + "/" + DateTime.Now.Year.ToString() + "</h1></b></font></td></tr>";
            else
                cadHtml += "<tr><td><font face='Verdana, Arial, Helvetica, sans-serif'><b><h1>Report saved at: " + DateTime.Now.Year.ToString() + "/" + DateTime.Now.Month.ToString() + "/" + DateTime.Now.Day.ToString() + "</h1></b></font></td></tr>";

            cadHtml += "</table><br><br>";

            for (int i = 0; i < aUrl.Count; i++)
            {
                if (aHost[i] == cGuardarUrls.Text || cGuardarUrls.Text == "Todas" || cGuardarUrls.Text == "All")
                {
                    cadHtml += "<table width='100%' border=1 cellpadding='5' cellspacing='5' bordercolor='#000000' bgcolor='#CCCCCC'>";
                    cadHtml += "<tr><td><font face='Verdana, Arial, Helvetica, sans-serif'><b><h1>" + aHost[i] + "</h1></b> (" + aUrl[i] + ")</font></td></tr>";

                    string en = formatearTextos.formatearHtml(aEnviado[i].Replace("&", "&amp;").Replace("<", "&lt;").Replace(">", "&gt;<br>").Replace("\r\n", "<br>"));
                    string ca = formatearTextos.formatearHtml(aCabecera[i].Replace("&", "&amp;").Replace("<", "&lt;").Replace(">", "&gt;<br>").Replace("\r\n", "<br>"));
                    string cu = formatearTextos.formatearHtml(aCuerpo[i].Replace("&", "&amp;").Replace("<", "&lt;").Replace(">", "&gt;<br>").Replace("\r\n", "<br>"));

                    if (Idioma == "es")
                    {
                        cadHtml += "<tr><td bgcolor='#999999'><font face='Verdana, Arial, Helvetica, sans-serif'><b><u>Datos enviados:</u></b></font></td></tr>";
                        cadHtml += "<tr><td><font face='Verdana, Arial, Helvetica, sans-serif'>" + en + "</font></td></tr>";
                        cadHtml += "<tr><td bgcolor='#999999'><font face='Verdana, Arial, Helvetica, sans-serif'><b><u>Datos recibidos</u></b></font></td></tr>";
                        cadHtml += "<tr><td bgcolor='#999999'><font face='Verdana, Arial, Helvetica, sans-serif'><u>Cabecera:</u></font></td></tr>";
                        cadHtml += "<tr><td><font face='Verdana, Arial, Helvetica, sans-serif'>" + ca + "</font></td></tr>";
                        cadHtml += "<tr><td bgcolor='#999999'><font face='Verdana, Arial, Helvetica, sans-serif'><u>Cuerpo:</u></font></td></tr>";
                        cadHtml += "<tr><td><font face='Verdana, Arial, Helvetica, sans-serif'>" + cu + "</font></td></tr>";
                    }
                    else
                    {
                        cadHtml += "<tr><td bgcolor='#999999'><font face='Verdana, Arial, Helvetica, sans-serif'><b><u>Data send:</u></b></font></td></tr>";
                        cadHtml += "<tr><td><font face='Verdana, Arial, Helvetica, sans-serif'>" + en + "</font></td></tr>";
                        cadHtml += "<tr><td bgcolor='#999999'><font face='Verdana, Arial, Helvetica, sans-serif'><b><u>Data received</u></b></font></td></tr>";
                        cadHtml += "<tr><td bgcolor='#999999'><font face='Verdana, Arial, Helvetica, sans-serif'><u>Header:</u></font></td></tr>";
                        cadHtml += "<tr><td><font face='Verdana, Arial, Helvetica, sans-serif'>" + ca + "</font></td></tr>";
                        cadHtml += "<tr><td bgcolor='#999999'><font face='Verdana, Arial, Helvetica, sans-serif'><u>Body:</u></font></td></tr>";
                        cadHtml += "<tr><td><font face='Verdana, Arial, Helvetica, sans-serif'>" + cu + "</font></td></tr>";
                    }

                    cadHtml += "</table><br>";
                }
            }

            cadHtml += "</body></html>";

            StreamWriter sw = new StreamWriter(fichero);
            sw.WriteLine(cadHtml);
            sw.Close();

            if (Idioma == "es")
                MessageBox.Show("Se ha guardado el fichero", "Guardar datos", MessageBoxButtons.OK, MessageBoxIcon.Information);
            else
                MessageBox.Show("Data file saved correctly", "Save data", MessageBoxButtons.OK, MessageBoxIcon.Information);

            Cursor.Current = Cursors.Default;
        }
    }
}