using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Xml;
using System.Web.UI;
using PGA_Seo;
using System.Threading;
using System.Text.RegularExpressions;
using SEOTool;
using System.Net;

namespace WindowsProxy
{
    public partial class FormSpider : Form
    {
        private delegate void updateUIAB();
        private delegate void updateUI(string pagina);
        private delegate void updateUIL(string texto);
        private delegate void updateUIR(string pagina, bool fin);
        private delegate void updateUIAN(TreeNode[] x, string key, string dato);
        private delegate void updateUIBN(string key);
        private Dictionary<string, string> dictHrefs = new Dictionary<string, string>();

        protected Regex m_rxBaseHref;
        protected Regex m_rxHref;
        protected Regex m_rxFrame;
        protected Regex m_rxIframe;
        protected Regex m_rxArea;
        protected Regex m_rxForm;
        protected Regex m_rxInput;

        protected List<string> UrlsSeguidas;
        protected List<string> Revisar;

        protected List<string> UrlName;
        protected List<string> UrlTitle;
        protected List<string> UrlDescription;
        protected List<string> UrlKeyword;

        protected List<string> LinkError;
        protected List<string> LinkInterno;
        protected List<string> LinkExterno;

        protected Thread tareaSpider = null;

        private List<string> google;
        bool esGoogle = false;

        String cUrl;
        String idioma;
        String cProxy;

        public String CProxy
        {
            get { return cProxy; }
            set { cProxy = value; }
        }


        public String Idioma
        {
            get { return idioma; }
            set { idioma = value; }
        }

        public String CUrl
        {
            get { return cUrl; }
            set { cUrl = value; }
        }

        public FormSpider()
        {
            InitializeComponent();

            RegexOptions rxOpt = RegexOptions.Singleline | RegexOptions.Compiled | RegexOptions.IgnoreCase;

            // Expresiones regulares
            m_rxHref = new Regex("<a[^>]*href=(\"|')(.*?)\\1[^>]*>(.*?)</a>", rxOpt);
            m_rxFrame = new Regex("<frame[^>]*src=(\"|')(.*?)\\1[^>]*>", rxOpt);
            m_rxIframe = new Regex("<iframe[^>]*src=(\"|')(.*?)\\1[^>]*>", rxOpt);
            m_rxArea = new Regex("<area[^>]*href=(\"|')(.*?)\\1[^>]*>", rxOpt);
            m_rxBaseHref = new Regex("<base[^>]* href=(\"|')(.*?)\\1[^>]*>", rxOpt);
            m_rxForm = new Regex("<form[^>]* action=(\"|')(.*?)\\1[^>]*>", rxOpt);
            m_rxInput = new Regex("<input[^>]* name=(\"|')(.*?)\\1[^>]*>", rxOpt);

            Revisar = new List<string>();
            Revisar.Add(".php");
            Revisar.Add(".php3");
            Revisar.Add(".php4");
            Revisar.Add(".php5");
            Revisar.Add(".asp");
            Revisar.Add(".aspx");
            Revisar.Add(".asmx");
            Revisar.Add(".jsp");
            Revisar.Add(".htm");
            Revisar.Add(".html");
            Revisar.Add(".shtml");
            Revisar.Add(".xml");
            Revisar.Add("");

            this.Closing += new CancelEventHandler(Form_Closing);
        }

        public void Form_Closing(object sender, CancelEventArgs cArgs)
        {
            if (sender == this)
            {
                try { tareaSpider.Abort(); }
                catch { }
            }
        }

        private void FinalizaEscaneo()
        {
            if (this.InvokeRequired == false)
            {
                int h = DateTime.Now.Hour;
                int m = DateTime.Now.Minute;
                int s = DateTime.Now.Second;
                String hor = h.ToString();
                String min = m.ToString();
                String sec = s.ToString();

                if (hor.Length == 1) hor = "0" + hor;
                if (min.Length == 1) min = "0" + min;
                if (sec.Length == 1) sec = "0" + sec;

                cLog.Text = hor + ":" + min + ":" + sec + " => ---------- FIN ----------" + Environment.NewLine + cLog.Text;

                bComenzar.Enabled = true;
                bCancelar.Enabled = false;

                progressBar1.Visible = false;

                if (idioma == "es")
                    MessageBox.Show("Se ha terminado el anlisis", this.Text, MessageBoxButtons.OK, MessageBoxIcon.Information, MessageBoxDefaultButton.Button1);
                else
                    MessageBox.Show("Analysis finished", this.Text, MessageBoxButtons.OK, MessageBoxIcon.Information, MessageBoxDefaultButton.Button1);
            }
            else
            {
                try
                {
                    updateUIAB updateMethod = new updateUIAB(this.FinalizaEscaneo);
                    this.Invoke(updateMethod, new object[] { });
                }
                catch { }
            }
        }

        private void bVolver_Click(object sender, EventArgs e)
        {
            try { tareaSpider.Abort(); }
            catch { }

            Close();
        }

        private void FormSpider_Load(object sender, EventArgs e)
        {
            cHost.Text = cUrl;

            bNavegar.Enabled = false;
            progressBar1.Visible = false;
            bComenzar.Enabled = true;
            bCancelar.Enabled = false;

            if (cHost.Text != "")
                ComenzarSpider();

            ConfiguraIdioma();
        }

        private void bNavegar_Click(object sender, EventArgs e)
        {
            FormNavegador fNavegar;
            fNavegar = new FormNavegador();
            fNavegar.Url = cRuta.Text;
            fNavegar.Show();
        }

        private void bComenzar_Click(object sender, EventArgs e)
        {
            cRuta.Text = "";
            esGoogle = false;
            ComenzarSpider();
        }

        private void ComenzarSpider()
        {
            progressBar1.Style = ProgressBarStyle.Marquee;

            if (cHost.Text == "")
            {
                if (idioma == "es")
                    MessageBox.Show("Tienes que escribie la Url que quieres escanear", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                else
                    MessageBox.Show("You have to write the Url you want to scan", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                cUrl = cHost.Text;
                cUrl = cUrl.Replace("http://", "");

                try
                {
                    if (tareaSpider != null)
                    {
                        tareaSpider.Abort();
                        tareaSpider.Join();
                    }

                    bComenzar.Enabled = false;
                    bCancelar.Enabled = true;
                    progressBar1.Visible = true;
                    tree.Nodes.Clear();
                    cLog.Text = "";

                    UrlsSeguidas = new List<string>();
                    UrlName = new List<string>();
                    UrlTitle = new List<string>();
                    UrlDescription = new List<string>();
                    UrlKeyword = new List<string>();
                    LinkError = new List<string>();
                    LinkInterno = new List<string>();
                    LinkExterno = new List<string>();

                    tree.Nodes.Add("/", "/", 1);

                    tareaSpider = new Thread(new ThreadStart(this.Analizar));
                    tareaSpider.Start();
                    //Analizar();
                }
                catch { }
            }
        }

        private void bCancelar_Click(object sender, EventArgs e)
        {
            try
            {
                tareaSpider.Abort();
            }
            catch { }

            progressBar1.Visible = false;
            bComenzar.Enabled = true;
            bCancelar.Enabled = false;
        }

        private void AnadirUrl(string pagina)
        {
            if (this.InvokeRequired == false)
            {
                int h = DateTime.Now.Hour;
                int m = DateTime.Now.Minute;
                int s = DateTime.Now.Second;
                string hor = h.ToString();
                string min = m.ToString();
                string sec = s.ToString();

                if (hor.Length == 1) hor = "0" + hor;
                if (min.Length == 1) min = "0" + min;
                if (sec.Length == 1) sec = "0" + sec;

                if (pagina.Contains("Formateando texto"))
                    cLog.Text = hor + ":" + min + ":" + sec + " => " + pagina + Environment.NewLine + cLog.Text;
                else
                    cLog.Text = hor + ":" + min + ":" + sec + " => Analizando " + pagina + Environment.NewLine + cLog.Text;
            }
            else
            {
                updateUI updateMethod = new updateUI(this.AnadirUrl);
                this.Invoke(updateMethod, new object[] { pagina });
            }
        }

        private void ActualizaLog(string texto)
        {
            if (this.InvokeRequired == false)
            {
                int h = DateTime.Now.Hour;
                int m = DateTime.Now.Minute;
                int s = DateTime.Now.Second;
                string hor = h.ToString();
                string min = m.ToString();
                string sec = s.ToString();

                if (hor.Length == 1) hor = "0" + hor;
                if (min.Length == 1) min = "0" + min;
                if (sec.Length == 1) sec = "0" + sec;

                cLog.Text = hor + ":" + min + ":" + sec + " => " + texto + Environment.NewLine + cLog.Text;
            }
            else
            {
                updateUIL updateMethod = new updateUIL(this.ActualizaLog);
                this.Invoke(updateMethod, new object[] { texto });
            }
        }

        private void AnadirNodo(TreeNode[] y, string key, string dato)
        {
            if (this.InvokeRequired == false)
            {
                tree.SelectedNode = y[0];

                //if (dato.Contains("."))
                if (key[key.Length - 1] != '/')
                {
                    if (esGoogle == true)
                        tree.SelectedNode.Nodes.Add(key, dato, 4);
                    else
                    {
                        if (Conexiones.Get("http://" + cUrl + key, cCookie.Text ,"") != null)
                            tree.SelectedNode.Nodes.Add(key, dato, 3);
                        else
                            tree.SelectedNode.Nodes.Add(key, dato, 2);
                    }
                }
                else
                    tree.SelectedNode.Nodes.Add(key, dato, 1);

                try { tree.Sort(); }
                catch { }

                try { tree.ExpandAll(); }
                catch { }
            }
            else
            {
                updateUIAN updateMethod = new updateUIAN(this.AnadirNodo);
                this.Invoke(updateMethod, new object[] { y, key, dato });
            }
        }

        private void BorrarNodo(string key)
        {
            if (this.InvokeRequired == false)
            {
                try
                {
                    tree.Nodes.RemoveByKey(key);
                    tree.Sort();
                }
                catch { }
            }
            else
            {
                updateUIBN updateMethod = new updateUIBN(this.BorrarNodo);
                this.Invoke(updateMethod, new object[] { key });
            }
        }

        private void Analizar()
        {
            Obtener("/");

            string pagina = Conexiones.Get("http://" + cUrl + "/robots.txt", cCookie.Text, cProxy);
            string ruta = "";
            int x = 0;

            if (pagina != null)
            {
                this.AnadirUrl("/robots.txt");

                while (x > -1)
                {
                    x = pagina.IndexOf("llow:");

                    if (x > -1)
                    {
                        pagina = pagina.Substring(x + 5);
                        x = pagina.IndexOf("\n");

                        if (x > -1)
                            ruta = pagina.Substring(0, x).Replace("\r", "");
                        else
                            ruta = pagina.Replace("\r", "");

                        ruta = ruta.Replace(" ", "");
                        if (ruta.Length < 1 || ruta[ruta.Length - 1] != '/') ruta += "/";

                        Obtener(ruta);
                    }
                }
            }

            BuscarEnGoogle();
            this.FinalizaEscaneo();
            tareaSpider.Abort();
        }

        #region Buscar en Google
        private void BuscarEnGoogle()
        {
            this.ActualizaLog("Realizando bsqueda en google.com");
            GetPosicionGoogle(cUrl, "www.google.com");
        }

        private void GetPosicionGoogle(string host, string buscador)
        {
            Int32 cont = 0;
            bool encontrado = true;

            google = new List<string>();

            while (encontrado == true)
            {
                encontrado = AnalizarGoogle(host, buscador, cont);
                cont += 10;
            }

            esGoogle = true;

            foreach (string pagina in google)
            {
                string tmp = pagina.Replace(CUrl, "").Replace("http://", "").Replace("www.", "").Replace(cUrl, "");
                if (tmp[0] != '/') tmp = "/" + tmp;
                int i = tmp.LastIndexOf("/" + 1);
                if (i < 0) i = 0;
                string ruta = tmp.Substring(0, i + 1);
                tmp = tmp.Substring(tmp.LastIndexOf("/"));
                this.AnadirUrl(tmp);

                if (!UrlsSeguidas.Contains(ruta + tmp.Substring(1)))
                {
                    TreeNode[] y = tree.Nodes.Find(ruta, true);
                    this.AnadirNodo(y, tmp, tmp.Substring(1));
                }
            }
        }

        private bool AnalizarGoogle(string host, string buscador, int posIni)
        {
            bool encontrado = false;
            int posIniAnt = posIni - 10;

            if (posIniAnt < 0) posIniAnt = 0;

            host = host.Replace("www.", "");
            String url = "http://" + buscador + "/ie?q=site:" + host + "&hl=en&pwst=1&start=" + posIni + "&sa=N";
            String referer = "http://" + buscador + "/ie?q=site:" + host + "&hl=en&pwst=1&start=" + posIniAnt + "&sa=N";
            HttpWebRequest wrGETURL = (HttpWebRequest)WebRequest.Create(url);
            wrGETURL.Referer = referer;
            wrGETURL.Headers.Set("Accept-Language", "en-US; en");
            wrGETURL.UserAgent = "AmpClient";

            try
            {
                Stream objStream = wrGETURL.GetResponse().GetResponseStream();
                StreamReader objReader = new StreamReader(objStream);

                String sLine = "";
                String cad = "";

                while (sLine != null)
                {
                    sLine = objReader.ReadLine();

                    if (sLine != null)
                        cad += sLine + Environment.NewLine;
                }

                encontrado = BuscarCoincidencias(cad);
            }
            catch { return false; }

            return encontrado;
        }

        private bool BuscarCoincidencias(String cad)
        {
            String host1 = cUrl.Replace("www.", "");

            if (!host1.EndsWith("/")) host1 += "/";

            String host2 = "http://www." + host1;
            host1 = "http://" + host1;

            int x = 0;
            int y1 = 0;
            int y2 = 0;

            while (y1 > -1)
            {
                y1 = cad.IndexOf(host1, x);
                y2 = cad.IndexOf(host2, x);

                if ((y2 > -1) && (y2 < y1 || y1 < 0)) y1 = y2;

                if (y2 > -1 || y1 > -1)
                {
                    int z = cad.IndexOf('>', y1);

                    if (z > -1)
                    {
                        string tmp = cad.Substring(y1, z - y1);
                        string extension = tmp.Substring(tmp.LastIndexOf("."));

                        if (!google.Contains(tmp) && Revisar.Contains(extension)) google.Add(tmp);
                    }

                    x = z;
                }
            }

            if (cad.Trim().ToLower().Contains(">next"))
                return true;
            else
                return false;
        }
        #endregion

        private bool Obtener(string directorio)
        {
            string dir = directorio;
            string pagina = Conexiones.Get("http://" + cUrl + directorio, cCookie.Text, "");

            if ((pagina == null) && (directorio == "/"))
            {
                pagina = Conexiones.Get("http://www." + cUrl + directorio, cCookie.Text, "");

                if (pagina != null)
                    cUrl = "www." + cUrl;
            }

            dictHrefs = new Dictionary<string, string>();
            directorio = directorio.Substring(0, directorio.LastIndexOf('/') + 1);

            if (pagina != null && pagina != "")
            {               
                MatchCollection matchesHref = m_rxHref.Matches(pagina);
                AddHrefMatches(dir, directorio, matchesHref, ref dictHrefs, pagina);

                MatchCollection matchesFrame = m_rxFrame.Matches(pagina);
                AddHrefMatches(dir, directorio, matchesFrame, ref dictHrefs, pagina);

                MatchCollection matchesIframe = m_rxIframe.Matches(pagina);
                AddHrefMatches(dir, directorio, matchesIframe, ref dictHrefs, pagina);

                MatchCollection matchesArea = m_rxArea.Matches(pagina);
                AddHrefMatches(dir, directorio, matchesArea, ref dictHrefs, pagina);

                MatchCollection matchesForm = m_rxForm.Matches(pagina);
                AddHrefMatches(dir, directorio, matchesForm, ref dictHrefs, pagina);

                Comprobar(dir, directorio, dictHrefs);

                return true;
            }
            else
            {
                if (dir[dir.Length - 1] != '/')
                    this.BorrarNodo(dir);

                return false;
            }
        }

        private void Comprobar(string origen, string directorio, Dictionary<string, string> dict)
        {
            foreach (string p in dict.Keys)
            {
                string ruta = p;

                if (!ruta.Contains(".")) ruta += "/";

                ruta = ruta.Replace("//", "/");

                if (!ruta.ToLower().Contains("http://") || (ruta.ToLower().Contains("http://") && ruta.Contains(cUrl)))
                {
                    if (!ruta.ToLower().Contains("mailto") && !ruta.Contains("#") && !UrlsSeguidas.Contains(ruta))
                    {
                        this.AnadirUrl(ruta);
                        Actualizar(ruta);
                        UrlsSeguidas.Add(ruta);

                        if (Obtener(ruta) == false)
                            if (ruta != "/\\/\\/\\/")
                                LinkError.Add(origen + " -> " + ruta);
                    }
                }
            }

            if (dict.Keys.Count == 0)
            {
                this.AnadirUrl(directorio);
                Actualizar(directorio);
                UrlsSeguidas.Add(directorio);
            }
        }

        protected void AddHrefMatches(string origen, string directorio, MatchCollection matches, ref Dictionary<string, string> dictHrefs, string pagina)
        {
            foreach (Match match in matches)
            {
                string ruta = match.Groups[2].Value.Replace("http://" + cUrl, "");
                ruta = ruta.Replace("http://" + cUrl.Replace("www.", ""), "");
                ruta = ruta.Replace("http://www." + cUrl.Replace("www.", ""), "");
                ruta = ruta.Replace("/?", "/index.php?");

                if (origen != "/")
                    if (ruta.Contains("http://") || ruta.Contains("https://"))
                        LinkExterno.Add(origen + " -> " + match.Groups[2].Value.Replace("http://" + cUrl, "") + " (" + match.Groups[3].Value.Trim() + ")");
                    else
                        LinkInterno.Add(origen + " -> " + match.Groups[2].Value.Replace("http://" + cUrl, "") + " (" + match.Groups[3].Value.Trim() + ")");

                if (ruta.StartsWith(directorio))
                    ruta = ruta.Remove(0, directorio.Length);
                if (directorio.Length > 1 && ruta.StartsWith(directorio.Substring(0, directorio.Length - 1)))
                {
                    int i = directorio.Length;

                    if (ruta.Length > i && ruta[i] == '/')
                        ruta = ruta.Remove(0, directorio.Length - 1);
                }

                if (ruta.Contains("#"))
                    ruta = ruta.Remove(ruta.IndexOf("#"));

                if (!ruta.ToLower().Contains("javascript") && !ruta.ToLower().Contains("http://") && ruta.Length > 0)
                {
                    string dir = directorio;

                    while (ruta.Contains("../"))
                    {
                        if (dir == "") dir = "/";

                        if ((dir[dir.Length - 1] == '/') && (dir != "/"))
                            dir = dir.Substring(0, dir.Length - 1);

                        dir = dir.Substring(0, dir.LastIndexOf("/"));
                        ruta = ruta.Remove(ruta.IndexOf("../"), 3);
                    }

                    dir += "/";
                    ruta = ruta.Replace("//", "/");
                    ruta = ruta.Replace("./", "");

                    if (!ruta.Contains(".") && ruta.Contains("?")) ruta = "index.php" + ruta;

                    if (!ruta.StartsWith("/")) ruta = dir + ruta;

                    if (!ruta.Contains(".")) ruta += "/";
                    ruta = ruta.Replace("//", "/");
                    ruta = ruta.Replace("&amp;", "&");

                    if (ruta.Contains("?"))
                        ruta = ruta.Substring(0, ruta.IndexOf("?"));

                    string extension = "";

                    if (ruta.Contains("."))
                        extension = ruta.Substring(ruta.LastIndexOf("."));

                    if (!dictHrefs.ContainsKey(ruta) && !UrlsSeguidas.Contains(ruta) && Revisar.Contains(extension))
                    {
                        if (match.Groups.Count > 3)
                            dictHrefs.Add(ruta, match.Groups[3].Value);
                        else
                            dictHrefs.Add(ruta, "");
                    }
                }
            }
        }

        protected string AddFormMatches(MatchCollection matches)
        {
            string rutaVuln = "";

            foreach (Match match in matches)
            {
                string name = match.Groups[2].Value;
                string value = "";
                int x = match.Value.IndexOf("value=");

                if (x > -1)
                {
                    value = match.Value.Substring(x + 7);
                    x = value.IndexOf("\"");
                    value = value.Substring(0, x);
                }

                if (rutaVuln != "") rutaVuln += "&";

                rutaVuln += name + "=" + value;
            }

            return rutaVuln;
        }

        private void Actualizar(string directorio)
        {
            string dir = "";
            directorio = directorio.Substring(1);
            string raiz = "/";

            TreeNode[] x = tree.Nodes.Find(directorio, true);

            if (directorio.Length > 0 && x.Length == 0)
            {
                while (directorio.Contains("/"))
                {
                    dir = directorio.Substring(0, directorio.IndexOf("/"));
                    x = tree.Nodes.Find(raiz + dir + "/", true);

                    if (x.Length == 0)
                    {
                        TreeNode[] y = tree.Nodes.Find(raiz, true);
                        if (dir != ("\\"))
                            this.AnadirNodo(y, raiz + dir + "/", dir);
                    }

                    raiz += dir + "/";
                    directorio = directorio.Substring(directorio.IndexOf("/") + 1);
                }

                if (directorio.Length > 0)
                {
                    TreeNode[] y = tree.Nodes.Find(raiz, true);
                    this.AnadirNodo(y, raiz + directorio, directorio);
                }
            }
        }

        private void tree_AfterSelect(object sender, TreeViewEventArgs e)
        {
            string fichero = "";

            fichero = tree.SelectedNode.Name.ToString();
            cRuta.Text = "http://" + cUrl + fichero;
        }

        private void cRuta_TextChanged(object sender, EventArgs e)
        {
            if (cRuta.Text.Length > 0)
                bNavegar.Enabled = true;
            else
                bNavegar.Enabled = false;
        }

        private void ConfiguraIdioma()
        {
            if (Idioma == "es")
            {
                bVolver.Text = "VOLVER";
                bComenzar.Text = "COMENZAR";
                bCancelar.Text = "PARAR";
                bNavegar.Text = "NAVEGAR";
                bGuardar.Text = "GUARDAR";
                bCargar.Text = "CARGAR";
                ldirectorio.Text = "Directorio";
                lok.Text = "Pina correcta";
                lerror.Text = "Pgina no encontrada";
                lgog.Text = "Slo en Google";
            }
            else
            {
                bVolver.Text = "BACK";
                bComenzar.Text = "START";
                bCancelar.Text = "STOP";
                bNavegar.Text = "NAVIGATE";
                bGuardar.Text = "SAVE";
                bCargar.Text = "LOAD";
                ldirectorio.Text = "Directory";
                lok.Text = "Correct page";
                lerror.Text = "Page not found";
                lgog.Text = "Only in Google";
            }
        }

        private void bGuardar_Click(object sender, EventArgs e)
        {
            if (tree.Nodes.Count > 0)
            {
                string rutaUrl = cUrl.Replace("/", "_");
                saveFileDialog1.FileName = rutaUrl + ".xml";
                saveFileDialog1.InitialDirectory = Path.GetDirectoryName(Application.ExecutablePath) + "\\data\\";
                saveFileDialog1.ShowDialog();
            }
            else
            {
                if (Idioma == "es")
                    MessageBox.Show("No hay datos que guardar", "Guardar datos", MessageBoxButtons.OK, MessageBoxIcon.Information);
                else
                    MessageBox.Show("There is no data to save", "Save data", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        private void saveFileDialog1_FileOk(object sender, CancelEventArgs e)
        {
            String fichero = saveFileDialog1.FileName;

            if (File.Exists(fichero))
            {
                try { File.Delete(fichero); }
                catch
                {
                    if (Idioma == "es")
                        MessageBox.Show("El fichero est bloqueado y no se puede sobreescribir", "Guardar datos", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    else
                        MessageBox.Show("File is locked and can't overwrite it", "Save data", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
            }

            bool esVuln = false;
            string rutaVuln = "";
            XmlDocument xDoc = new XmlDocument();
            String cadXml = "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>";
            cadXml += "<data>";
            cadXml += "<host>" + cUrl + "</host>";
            cadXml += "<lista_urls>";

            foreach (string ruta in UrlsSeguidas)
            {
                esVuln = false;
                rutaVuln = "";

                cadXml += "<urls>";
                cadXml += "<url>" + ruta + "</url>";

                if (rutaVuln == "") rutaVuln = ruta;
                if (tree.Nodes.Find(ruta, true)[0].ImageIndex == 2)
                    esVuln = true;

                cadXml += "<ruta>![CDATA[" + rutaVuln.Replace("&", "#") + "]]</ruta>";
                cadXml += "<vulnerable>" + esVuln + "</vulnerable>";
                cadXml += "</urls>";
            }

            cadXml += "</lista_urls>";
            cadXml += "</data>";
            xDoc.LoadXml(cadXml);

            xDoc.Save(fichero);

            if (idioma == "es")
                MessageBox.Show("Se ha guardado el fichero", "Guardar datos", MessageBoxButtons.OK, MessageBoxIcon.Information);
            else
                MessageBox.Show("Data file saved correctly", "Save data", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void bCargar_Click(object sender, EventArgs e)
        {
            openFileDialog1.InitialDirectory = Path.GetDirectoryName(Application.ExecutablePath) + "\\data\\";
            openFileDialog1.ShowDialog();
        }

        private void openFileDialog1_FileOk(object sender, CancelEventArgs e)
        {
            Cursor.Current = Cursors.WaitCursor;

            XmlDocument xDoc = new XmlDocument();
            xDoc.Load(openFileDialog1.FileName);

            XmlNodeList host = xDoc.GetElementsByTagName("host");
            cHost.Text = ((XmlElement)host[0]).InnerText;
            cUrl = cHost.Text;

            XmlNodeList listaUrls = xDoc.GetElementsByTagName("lista_urls");
            XmlNodeList lista = ((XmlElement)listaUrls[0]).GetElementsByTagName("urls");

            UrlsSeguidas = new List<string>();

            tree.Nodes.Clear();
            tree.Nodes.Add("/", "/", 1);

            foreach (XmlElement nodo in lista)
            {
                //if (nodo["url"].InnerText != nodo["ruta"].InnerText.Replace("![CDATA[", "").Replace("]]", "").Replace("#", "&"))
                    UrlsSeguidas.Add(nodo["url"].InnerText);

                Actualizar(nodo["url"].InnerText);
            }

            cCookie.Text = "";

            Cursor.Current = Cursors.Default;
        }

        private void cHost_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == 13)
                bComenzar_Click(sender, e);
        }

        private void bMas_Click(object sender, EventArgs e)
        {
            tree.ExpandAll();
        }

        private void bMenos_Click(object sender, EventArgs e)
        {
            tree.CollapseAll();
        }

        private void bExportar_Click(object sender, EventArgs e)
        {
            if (tree.Nodes.Count > 0)
            {
                string rutaUrl = cUrl.Replace("/", "_");
                saveFileDialog2.FileName = rutaUrl + ".html";
                saveFileDialog2.InitialDirectory = Path.GetDirectoryName(Application.ExecutablePath) + "\\data\\";
                saveFileDialog2.ShowDialog();
            }
            else
            {
                if (Idioma == "es")
                    MessageBox.Show("No hay datos que guardar", "Guardar datos", MessageBoxButtons.OK, MessageBoxIcon.Information);
                else
                    MessageBox.Show("There is no data to save", "Save data", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        private void saveFileDialog2_FileOk(object sender, CancelEventArgs e)
        {
            Cursor.Current = Cursors.WaitCursor;

            String fichero = saveFileDialog2.FileName;

            if (File.Exists(fichero))
            {
                try { File.Delete(fichero); }
                catch
                {
                    if (Idioma == "es")
                        MessageBox.Show("El fichero est bloqueado y no se puede sobreescribir", "Guardar datos", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    else
                        MessageBox.Show("File is locked and can't overwrite it", "Save data", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
            }

            String cadHtml = "<html><head>";
            cadHtml += "<title>" + cUrl + "</title></head>";
            cadHtml += "<body>";
            cadHtml += "<table width='100%' border=1 cellpadding='5' cellspacing='5' bordercolor='#000000' bgcolor='#CCCCCC'>";

            if (Idioma == "es")
                cadHtml += "<tr><td><font face='Verdana, Arial, Helvetica, sans-serif'><b><h1>Informe guardado el: " + DateTime.Now.Day.ToString() + "/" + DateTime.Now.Month.ToString() + "/" + DateTime.Now.Year.ToString() + "</h1></b></font></td></tr>";
            else
                cadHtml += "<tr><td><font face='Verdana, Arial, Helvetica, sans-serif'><b><h1>Report saved at: " + DateTime.Now.Year.ToString() + "/" + DateTime.Now.Month.ToString() + "/" + DateTime.Now.Day.ToString() + "</h1></b></font></td></tr>";

            cadHtml += "</table><br><br>";
            
            foreach (string ruta in UrlsSeguidas)
                if (ruta != "/")
                    cadHtml += "<p><font face='Verdana, Arial, Helvetica, sans-serif'>" + ruta + "</font></p>";

            cadHtml += "</body></html>";

            StreamWriter sw = new StreamWriter(fichero);
            sw.WriteLine(cadHtml);
            sw.Close();

            if (Idioma == "es")
                MessageBox.Show("Se ha guardado el fichero", "Guardar datos", MessageBoxButtons.OK, MessageBoxIcon.Information);
            else
                MessageBox.Show("Data file saved correctly", "Save data", MessageBoxButtons.OK, MessageBoxIcon.Information);

            Cursor.Current = Cursors.Default;
        }
    }
}